import sys
from _typeshed import ReadOnlyBuffer, SupportsRead, SupportsWrite
from curses import _ncurses_version
from typing import Any, Final, TypeAlias, final, overload

# NOTE: This module is ordinarily only available on Unix, but the windows-curses
# package makes it available on Windows as well with the same contents.

# Handled by PyCurses_ConvertToChtype in _cursesmodule.c.
_ChType: TypeAlias = str | bytes | int

# ACS codes are only initialized after initscr is called
ACS_BBSS: Final[int]
ACS_BLOCK: Final[int]
ACS_BOARD: Final[int]
ACS_BSBS: Final[int]
ACS_BSSB: Final[int]
ACS_BSSS: Final[int]
ACS_BTEE: Final[int]
ACS_BULLET: Final[int]
ACS_CKBOARD: Final[int]
ACS_DARROW: Final[int]
ACS_DEGREE: Final[int]
ACS_DIAMOND: Final[int]
ACS_GEQUAL: Final[int]
ACS_HLINE: Final[int]
ACS_LANTERN: Final[int]
ACS_LARROW: Final[int]
ACS_LEQUAL: Final[int]
ACS_LLCORNER: Final[int]
ACS_LRCORNER: Final[int]
ACS_LTEE: Final[int]
ACS_NEQUAL: Final[int]
ACS_PI: Final[int]
ACS_PLMINUS: Final[int]
ACS_PLUS: Final[int]
ACS_RARROW: Final[int]
ACS_RTEE: Final[int]
ACS_S1: Final[int]
ACS_S3: Final[int]
ACS_S7: Final[int]
ACS_S9: Final[int]
ACS_SBBS: Final[int]
ACS_SBSB: Final[int]
ACS_SBSS: Final[int]
ACS_SSBB: Final[int]
ACS_SSBS: Final[int]
ACS_SSSB: Final[int]
ACS_SSSS: Final[int]
ACS_STERLING: Final[int]
ACS_TTEE: Final[int]
ACS_UARROW: Final[int]
ACS_ULCORNER: Final[int]
ACS_URCORNER: Final[int]
ACS_VLINE: Final[int]
ALL_MOUSE_EVENTS: Final[int]
A_ALTCHARSET: Final[int]
A_ATTRIBUTES: Final[int]
A_BLINK: Final[int]
A_BOLD: Final[int]
A_CHARTEXT: Final[int]
A_COLOR: Final[int]
A_DIM: Final[int]
A_HORIZONTAL: Final[int]
A_INVIS: Final[int]
A_ITALIC: Final[int]
A_LEFT: Final[int]
A_LOW: Final[int]
A_NORMAL: Final[int]
A_PROTECT: Final[int]
A_REVERSE: Final[int]
A_RIGHT: Final[int]
A_STANDOUT: Final[int]
A_TOP: Final[int]
A_UNDERLINE: Final[int]
A_VERTICAL: Final[int]
BUTTON1_CLICKED: Final[int]
BUTTON1_DOUBLE_CLICKED: Final[int]
BUTTON1_PRESSED: Final[int]
BUTTON1_RELEASED: Final[int]
BUTTON1_TRIPLE_CLICKED: Final[int]
BUTTON2_CLICKED: Final[int]
BUTTON2_DOUBLE_CLICKED: Final[int]
BUTTON2_PRESSED: Final[int]
BUTTON2_RELEASED: Final[int]
BUTTON2_TRIPLE_CLICKED: Final[int]
BUTTON3_CLICKED: Final[int]
BUTTON3_DOUBLE_CLICKED: Final[int]
BUTTON3_PRESSED: Final[int]
BUTTON3_RELEASED: Final[int]
BUTTON3_TRIPLE_CLICKED: Final[int]
BUTTON4_CLICKED: Final[int]
BUTTON4_DOUBLE_CLICKED: Final[int]
BUTTON4_PRESSED: Final[int]
BUTTON4_RELEASED: Final[int]
BUTTON4_TRIPLE_CLICKED: Final[int]
# Darwin ncurses doesn't provide BUTTON5_* constants prior to 3.12.10 and 3.13.3
if sys.version_info >= (3, 12) or sys.platform != "darwin":
    BUTTON5_PRESSED: Final[int]
    BUTTON5_RELEASED: Final[int]
    BUTTON5_CLICKED: Final[int]
    BUTTON5_DOUBLE_CLICKED: Final[int]
    BUTTON5_TRIPLE_CLICKED: Final[int]
BUTTON_ALT: Final[int]
BUTTON_CTRL: Final[int]
BUTTON_SHIFT: Final[int]
COLOR_BLACK: Final[int]
COLOR_BLUE: Final[int]
COLOR_CYAN: Final[int]
COLOR_GREEN: Final[int]
COLOR_MAGENTA: Final[int]
COLOR_RED: Final[int]
COLOR_WHITE: Final[int]
COLOR_YELLOW: Final[int]
ERR: Final[int]
KEY_A1: Final[int]
KEY_A3: Final[int]
KEY_B2: Final[int]
KEY_BACKSPACE: Final[int]
KEY_BEG: Final[int]
KEY_BREAK: Final[int]
KEY_BTAB: Final[int]
KEY_C1: Final[int]
KEY_C3: Final[int]
KEY_CANCEL: Final[int]
KEY_CATAB: Final[int]
KEY_CLEAR: Final[int]
KEY_CLOSE: Final[int]
KEY_COMMAND: Final[int]
KEY_COPY: Final[int]
KEY_CREATE: Final[int]
KEY_CTAB: Final[int]
KEY_DC: Final[int]
KEY_DL: Final[int]
KEY_DOWN: Final[int]
KEY_EIC: Final[int]
KEY_END: Final[int]
KEY_ENTER: Final[int]
KEY_EOL: Final[int]
KEY_EOS: Final[int]
KEY_EXIT: Final[int]
KEY_F0: Final[int]
KEY_F1: Final[int]
KEY_F10: Final[int]
KEY_F11: Final[int]
KEY_F12: Final[int]
KEY_F13: Final[int]
KEY_F14: Final[int]
KEY_F15: Final[int]
KEY_F16: Final[int]
KEY_F17: Final[int]
KEY_F18: Final[int]
KEY_F19: Final[int]
KEY_F2: Final[int]
KEY_F20: Final[int]
KEY_F21: Final[int]
KEY_F22: Final[int]
KEY_F23: Final[int]
KEY_F24: Final[int]
KEY_F25: Final[int]
KEY_F26: Final[int]
KEY_F27: Final[int]
KEY_F28: Final[int]
KEY_F29: Final[int]
KEY_F3: Final[int]
KEY_F30: Final[int]
KEY_F31: Final[int]
KEY_F32: Final[int]
KEY_F33: Final[int]
KEY_F34: Final[int]
KEY_F35: Final[int]
KEY_F36: Final[int]
KEY_F37: Final[int]
KEY_F38: Final[int]
KEY_F39: Final[int]
KEY_F4: Final[int]
KEY_F40: Final[int]
KEY_F41: Final[int]
KEY_F42: Final[int]
KEY_F43: Final[int]
KEY_F44: Final[int]
KEY_F45: Final[int]
KEY_F46: Final[int]
KEY_F47: Final[int]
KEY_F48: Final[int]
KEY_F49: Final[int]
KEY_F5: Final[int]
KEY_F50: Final[int]
KEY_F51: Final[int]
KEY_F52: Final[int]
KEY_F53: Final[int]
KEY_F54: Final[int]
KEY_F55: Final[int]
KEY_F56: Final[int]
KEY_F57: Final[int]
KEY_F58: Final[int]
KEY_F59: Final[int]
KEY_F6: Final[int]
KEY_F60: Final[int]
KEY_F61: Final[int]
KEY_F62: Final[int]
KEY_F63: Final[int]
KEY_F7: Final[int]
KEY_F8: Final[int]
KEY_F9: Final[int]
KEY_FIND: Final[int]
KEY_HELP: Final[int]
KEY_HOME: Final[int]
KEY_IC: Final[int]
KEY_IL: Final[int]
KEY_LEFT: Final[int]
KEY_LL: Final[int]
KEY_MARK: Final[int]
KEY_MAX: Final[int]
KEY_MESSAGE: Final[int]
KEY_MIN: Final[int]
KEY_MOUSE: Final[int]
KEY_MOVE: Final[int]
KEY_NEXT: Final[int]
KEY_NPAGE: Final[int]
KEY_OPEN: Final[int]
KEY_OPTIONS: Final[int]
KEY_PPAGE: Final[int]
KEY_PREVIOUS: Final[int]
KEY_PRINT: Final[int]
KEY_REDO: Final[int]
KEY_REFERENCE: Final[int]
KEY_REFRESH: Final[int]
KEY_REPLACE: Final[int]
KEY_RESET: Final[int]
KEY_RESIZE: Final[int]
KEY_RESTART: Final[int]
KEY_RESUME: Final[int]
KEY_RIGHT: Final[int]
KEY_SAVE: Final[int]
KEY_SBEG: Final[int]
KEY_SCANCEL: Final[int]
KEY_SCOMMAND: Final[int]
KEY_SCOPY: Final[int]
KEY_SCREATE: Final[int]
KEY_SDC: Final[int]
KEY_SDL: Final[int]
KEY_SELECT: Final[int]
KEY_SEND: Final[int]
KEY_SEOL: Final[int]
KEY_SEXIT: Final[int]
KEY_SF: Final[int]
KEY_SFIND: Final[int]
KEY_SHELP: Final[int]
KEY_SHOME: Final[int]
KEY_SIC: Final[int]
KEY_SLEFT: Final[int]
KEY_SMESSAGE: Final[int]
KEY_SMOVE: Final[int]
KEY_SNEXT: Final[int]
KEY_SOPTIONS: Final[int]
KEY_SPREVIOUS: Final[int]
KEY_SPRINT: Final[int]
KEY_SR: Final[int]
KEY_SREDO: Final[int]
KEY_SREPLACE: Final[int]
KEY_SRESET: Final[int]
KEY_SRIGHT: Final[int]
KEY_SRSUME: Final[int]
KEY_SSAVE: Final[int]
KEY_SSUSPEND: Final[int]
KEY_STAB: Final[int]
KEY_SUNDO: Final[int]
KEY_SUSPEND: Final[int]
KEY_UNDO: Final[int]
KEY_UP: Final[int]
OK: Final[int]
REPORT_MOUSE_POSITION: Final[int]
_C_API: Any
version: Final[bytes]

def baudrate() -> int:
    """Return the output speed of the terminal in bits per second."""

def beep() -> None:
    """Emit a short attention sound."""

def can_change_color() -> bool:
    """Return True if the programmer can change the colors displayed by the terminal."""

def cbreak(flag: bool = True, /) -> None:
    """Enter cbreak mode.

      flag
        If false, the effect is the same as calling nocbreak().

    In cbreak mode (sometimes called "rare" mode) normal tty line buffering
    is turned off and characters are available to be read one by one.
    However, unlike raw mode, special characters (interrupt, quit, suspend,
    and flow control) retain their effects on the tty driver and calling
    program.  Calling first raw() then cbreak() leaves the terminal in
    cbreak mode.
    """

def color_content(color_number: int, /) -> tuple[int, int, int]:
    """Return the red, green, and blue (RGB) components of the specified color.

      color_number
        The number of the color (0 - (COLORS-1)).

    A 3-tuple is returned, containing the R, G, B values for the given
    color, which will be between 0 (no component) and 1000 (maximum amount
    of component).
    """

def color_pair(pair_number: int, /) -> int:
    """Return the attribute value for displaying text in the specified color.

      pair_number
        The number of the color pair.

    This attribute value can be combined with A_STANDOUT, A_REVERSE, and the
    other A_* attributes.  pair_number() is the counterpart to this
    function.
    """

def curs_set(visibility: int, /) -> int:
    """Set the cursor state.

      visibility
        0 for invisible, 1 for normal visible, or 2 for very visible.

    If the terminal supports the visibility requested, the previous cursor
    state is returned; otherwise, an exception is raised.  On many
    terminals, the "visible" mode is an underline cursor and the "very
    visible" mode is a block cursor.
    """

def def_prog_mode() -> None:
    """Save the current terminal mode as the "program" mode.

    The "program" mode is the mode when the running program is using curses.

    Subsequent calls to reset_prog_mode() will restore this mode.
    """

def def_shell_mode() -> None:
    """Save the current terminal mode as the "shell" mode.

    The "shell" mode is the mode when the running program is not using
    curses.

    Subsequent calls to reset_shell_mode() will restore this mode.
    """

def delay_output(ms: int, /) -> None:
    """Insert a pause in output.

    ms
      Duration in milliseconds.
    """

def doupdate() -> None:
    """Update the physical screen to match the virtual screen."""

def echo(flag: bool = True, /) -> None:
    """Enter echo mode.

      flag
        If false, the effect is the same as calling noecho().

    In echo mode, each character input is echoed to the screen as it is
    entered.
    """

def endwin() -> None:
    """De-initialize the library, and return terminal to normal status."""

def erasechar() -> bytes:
    """Return the user's current erase character."""

def filter() -> None:
    """Restrict screen updates to the current line.

    Must be called before initscr().  Afterwards curses confines the cursor
    and screen updates to a single line, which is useful for enabling
    character-at-a-time line editing without touching the rest of the
    screen.
    """

def flash() -> None:
    """Flash the screen.

    That is, change it to reverse-video and then change it back in a short
    interval.
    """

def flushinp() -> None:
    """Flush all input buffers.

    This throws away any typeahead that has been typed by the user and has
    not yet been processed by the program.
    """

def get_escdelay() -> int:
    """Gets the curses ESCDELAY setting.

    Gets the number of milliseconds to wait after reading an escape
    character, to distinguish between an individual escape character entered
    on the keyboard from escape sequences sent by cursor and function keys.
    """

def get_tabsize() -> int:
    """Gets the curses TABSIZE setting.

    Gets the number of columns used by the curses library when converting
    a tab character to spaces as it adds the tab to a window.
    """

def getmouse() -> tuple[int, int, int, int, int]:
    """Retrieve the queued mouse event.

    After getch() returns KEY_MOUSE to signal a mouse event, this function
    returns a 5-tuple (id, x, y, z, bstate).
    """

def getsyx() -> tuple[int, int]:
    """Return the current coordinates of the virtual screen cursor.

    Return a (y, x) tuple.  If leaveok is currently true, return (-1, -1).
    """

def getwin(file: SupportsRead[bytes], /) -> window:
    """Read window related data stored in the file by an earlier putwin() call.

    The routine then creates and initializes a new window using that data,
    returning the new window object.
    """

def halfdelay(tenths: int, /) -> None:
    """Enter half-delay mode.

      tenths
        Maximal blocking delay in tenths of seconds (1 - 255).

    Use nocbreak() to leave half-delay mode.
    """

def has_colors() -> bool:
    """Return True if the terminal can display colors; otherwise, return False."""

def has_extended_color_support() -> bool:
    """Return True if the module supports extended colors; otherwise, return False.

    Extended color support allows more than 256 color-pairs for terminals
    that support more than 16 colors (e.g. xterm-256color).
    """

if sys.version_info >= (3, 14):
    def assume_default_colors(fg: int, bg: int, /) -> None:
        """Allow use of default values for colors on terminals supporting this feature.

        Assign terminal default foreground/background colors to color number -1.
        Change the definition of the color-pair 0 to (fg, bg).

        Use this to support transparency in your application.
        """

def has_ic() -> bool:
    """Return True if the terminal has insert- and delete-character capabilities."""

def has_il() -> bool:
    """Return True if the terminal has insert- and delete-line capabilities."""

def has_key(key: int, /) -> bool:
    """Return True if the current terminal type recognizes a key with that value.

    key
      Key number.
    """

def init_color(color_number: int, r: int, g: int, b: int, /) -> None:
    """Change the definition of a color.

      color_number
        The number of the color to be changed (0 - (COLORS-1)).
      r
        Red component (0 - 1000).
      g
        Green component (0 - 1000).
      b
        Blue component (0 - 1000).

    When init_color() is used, all occurrences of that color on the screen
    immediately change to the new definition.  This function is a no-op on
    most terminals; it is active only if can_change_color() returns true.
    """

def init_pair(pair_number: int, fg: int, bg: int, /) -> None:
    """Change the definition of a color-pair.

      pair_number
        The number of the color-pair to be changed (1 - (COLOR_PAIRS-1)).
      fg
        Foreground color number (-1 - (COLORS-1)).
      bg
        Background color number (-1 - (COLORS-1)).

    If the color-pair was previously initialized, the screen is refreshed
    and all occurrences of that color-pair are changed to the new
    definition.
    """

def initscr() -> window:
    """Initialize the library.

    Return a WindowObject which represents the whole screen.
    """

def intrflush(flag: bool, /) -> None:
    """Control flushing of the output buffer when an interrupt key is pressed.

    If flag is true, pressing an interrupt key (interrupt, break, or quit)
    flushes all output in the terminal driver queue.  If flag is false, no
    flushing is done.
    """

def is_term_resized(nlines: int, ncols: int, /) -> bool:
    """Return True if resize_term() would modify the window structure, False otherwise.

    nlines
      Height.
    ncols
      Width.
    """

def isendwin() -> bool:
    """Return True if endwin() has been called."""

def keyname(key: int, /) -> bytes:
    """Return the name of specified key.

    key
      Key number.
    """

def killchar() -> bytes:
    """Return the user's current line kill character."""

def longname() -> bytes:
    """Return the terminfo long name field describing the current terminal.

    The maximum length of a verbose description is 128 characters.  It is
    defined only after the call to initscr().
    """

def meta(yes: bool, /) -> None:
    """Enable/disable meta keys.

    If yes is True, allow 8-bit characters to be input.  If yes is False,
    allow only 7-bit characters.
    """

def mouseinterval(interval: int, /) -> None:
    """Set and retrieve the maximum time between press and release in a click.

      interval
        Time in milliseconds.

    Set the maximum time that can elapse between press and release events in
    order for them to be recognized as a click, and return the previous
    interval value.
    """

def mousemask(newmask: int, /) -> tuple[int, int]:
    """Set the mouse events to be reported, and return (availmask, oldmask).

    Return a tuple (availmask, oldmask).  availmask indicates which of the
    specified mouse events can be reported; on complete failure it returns
    0.  oldmask is the previous value of the mouse event mask.  If this
    function is never called, no mouse events are ever reported.
    """

def napms(ms: int, /) -> int:
    """Sleep for specified time.

    ms
      Duration in milliseconds.
    """

def newpad(nlines: int, ncols: int, /) -> window:
    """Create and return a pointer to a new pad data structure.

    nlines
      Height.
    ncols
      Width.
    """

def newwin(nlines: int, ncols: int, begin_y: int = 0, begin_x: int = 0, /) -> window:
    """newwin(nlines, ncols, [begin_y=0, begin_x=0])
    Return a new window.

      nlines
        Height.
      ncols
        Width.
      begin_y
        Top side y-coordinate.
      begin_x
        Left side x-coordinate.

    By default, the window will extend from the specified position to the
    lower right corner of the screen.
    """

def nl(flag: bool = True, /) -> None:
    """Enter newline mode.

      flag
        If false, the effect is the same as calling nonl().

    This mode translates the return key into newline on input, and
    translates newline into return and line-feed on output.  Newline mode
    is initially on.
    """

def nocbreak() -> None:
    """Leave cbreak mode.

    Return to normal "cooked" mode with line buffering.
    """

def noecho() -> None:
    """Leave echo mode.

    Echoing of input characters is turned off.
    """

def nonl() -> None:
    """Leave newline mode.

    Disable translation of return into newline on input, and disable
    low-level translation of newline into newline/return on output.
    """

def noqiflush() -> None:
    """Disable queue flushing.

    When queue flushing is disabled, normal flush of input and output queues
    associated with the INTR, QUIT and SUSP characters will not be done.
    """

def noraw() -> None:
    """Leave raw mode.

    Return to normal "cooked" mode with line buffering.
    """

def pair_content(pair_number: int, /) -> tuple[int, int]:
    """Return a tuple (fg, bg) containing the colors for the requested color pair.

    pair_number
      The number of the color pair (0 - (COLOR_PAIRS-1)).
    """

def pair_number(attr: int, /) -> int:
    """Return the number of the color-pair set by the specified attribute value.

    color_pair() is the counterpart to this function.
    """

def putp(string: ReadOnlyBuffer, /) -> None:
    """Emit the value of a specified terminfo capability for the current terminal.

    Note that the output of putp() always goes to standard output.
    """

def qiflush(flag: bool = True, /) -> None:
    """Enable queue flushing.

      flag
        If false, the effect is the same as calling noqiflush().

    If queue flushing is enabled, all output in the display driver queue
    will be flushed when the INTR, QUIT and SUSP characters are read.
    """

def raw(flag: bool = True, /) -> None:
    """Enter raw mode.

      flag
        If false, the effect is the same as calling noraw().

    In raw mode, normal line buffering and processing of interrupt, quit,
    suspend, and flow control keys are turned off; characters are presented
    to curses input functions one by one.
    """

def reset_prog_mode() -> None:
    """Restore the terminal to "program" mode, as previously saved by def_prog_mode()."""

def reset_shell_mode() -> None:
    """Restore the terminal to "shell" mode, as previously saved by def_shell_mode()."""

def resetty() -> None:
    """Restore terminal mode."""

def resize_term(nlines: int, ncols: int, /) -> None:
    """Backend function used by resizeterm(), performing most of the work.

      nlines
        Height.
      ncols
        Width.

    When resizing the windows, resize_term() blank-fills the areas that are
    extended.  The calling application should fill in these areas with
    appropriate data.  The resize_term() function attempts to resize all
    windows.  However, due to the calling convention of pads, it is not
    possible to resize these without additional interaction with the
    application.
    """

def resizeterm(nlines: int, ncols: int, /) -> None:
    """Resize the standard and current windows to the specified dimensions.

      nlines
        Height.
      ncols
        Width.

    Adjusts other bookkeeping data used by the curses library that record
    the window dimensions (in particular the SIGWINCH handler).
    """

def savetty() -> None:
    """Save terminal mode."""

def set_escdelay(ms: int, /) -> None:
    """Sets the curses ESCDELAY setting.

      ms
        length of the delay in milliseconds.

    Sets the number of milliseconds to wait after reading an escape
    character, to distinguish between an individual escape character entered
    on the keyboard from escape sequences sent by cursor and function keys.
    """

def set_tabsize(size: int, /) -> None:
    """Sets the curses TABSIZE setting.

      size
        rendered cell width of a tab character.

    Sets the number of columns used by the curses library when converting
    a tab character to spaces as it adds the tab to a window.
    """

def setsyx(y: int, x: int, /) -> None:
    """Set the virtual screen cursor.

      y
        Y-coordinate.
      x
        X-coordinate.

    If y and x are both -1, then leaveok is set.
    """

def setupterm(term: str | None = None, fd: int = -1) -> None:
    """Initialize the terminal.

    term
      Terminal name.
      If omitted, the value of the TERM environment variable will be used.
    fd
      File descriptor to which any initialization sequences will be sent.
      If not supplied, the file descriptor for sys.stdout will be used.
    """

def start_color() -> None:
    """Initializes eight basic colors and global variables COLORS and COLOR_PAIRS.

    Must be called if the programmer wants to use colors, and before any
    other color manipulation routine is called.  It is good practice to call
    this routine right after initscr().

    It also restores the colors on the terminal to the values they had when
    the terminal was just turned on.
    """

def termattrs() -> int:
    """Return a logical OR of all video attributes supported by the terminal."""

def termname() -> bytes:
    """Return the value of the environment variable TERM, truncated to 14 characters."""

def tigetflag(capname: str, /) -> int:
    """Return the value of the Boolean capability.

      capname
        The terminfo capability name.

    The value -1 is returned if capname is not a Boolean capability, or 0 if
    it is canceled or absent from the terminal description.
    """

def tigetnum(capname: str, /) -> int:
    """Return the value of the numeric capability.

      capname
        The terminfo capability name.

    The value -2 is returned if capname is not a numeric capability, or -1
    if it is canceled or absent from the terminal description.
    """

def tigetstr(capname: str, /) -> bytes | None:
    """Return the value of the string capability.

      capname
        The terminfo capability name.

    None is returned if capname is not a string capability, or is canceled
    or absent from the terminal description.
    """

def tparm(
    str: ReadOnlyBuffer,
    i1: int = 0,
    i2: int = 0,
    i3: int = 0,
    i4: int = 0,
    i5: int = 0,
    i6: int = 0,
    i7: int = 0,
    i8: int = 0,
    i9: int = 0,
    /,
) -> bytes:
    """Instantiate the specified byte string with the supplied parameters.

    str
      Parameterized byte string obtained from the terminfo database.
    """

def typeahead(fd: int, /) -> None:
    """Specify that the file descriptor fd be used for typeahead checking.

      fd
        File descriptor.

    If fd is -1, then no typeahead checking is done.
    """

def unctrl(ch: _ChType, /) -> bytes:
    """Return a string which is a printable representation of the character ch.

    Control characters are displayed as a caret followed by the character,
    for example as ^C.  Printing characters are left as they are.
    """

def unget_wch(ch: int | str, /) -> None:
    """Push ch so the next get_wch() will return it."""

def ungetch(ch: _ChType, /) -> None:
    """Push ch so the next getch() will return it."""

def ungetmouse(id: int, x: int, y: int, z: int, bstate: int, /) -> None:
    """Push a KEY_MOUSE event onto the input queue.

    The following getmouse() will return the given state data.
    """

def update_lines_cols() -> None:
    """Update the LINES and COLS module variables.

    This is useful for detecting manual screen resize.
    """

def use_default_colors() -> None:
    """Equivalent to assume_default_colors(-1, -1)."""

def use_env(flag: bool, /) -> None:
    """Use environment variables LINES and COLUMNS.

    If used, this function should be called before initscr() or newterm()
    are called.

    When flag is False, the values of lines and columns specified in the
    terminfo database will be used, even if environment variables LINES and
    COLUMNS (used by default) are set, or if curses is running in a window
    (in which case default behavior would be to use the window size if LINES
    and COLUMNS are not set).
    """

class error(Exception):
    """Exception raised when a curses library function returns an error."""

@final
class window:  # undocumented
    """A curses window.

    Window objects are returned by initscr() and newwin(), and by the
    methods that create subwindows and pads.
    """

    encoding: str
    """the typecode character used to create the array"""

    @overload
    def addch(self, ch: _ChType, attr: int = ...) -> None:
        """addch([y, x,] ch, [attr=_curses.A_NORMAL])
        Paint the character.

          y
            Y-coordinate.
          x
            X-coordinate.
          ch
            Character to add.
          attr
            Attributes for the character.

        Paint character ch at (y, x) with attributes attr,
        overwriting any character previously painted at that location.
        By default, the character position and attributes are the
        current settings for the window object.
        """
    @overload
    def addch(self, y: int, x: int, ch: _ChType, attr: int = ...) -> None: ...

    @overload
    def addnstr(self, str: str, n: int, attr: int = ...) -> None:
        """addnstr([y, x,] str, n, [attr])
        Paint at most n characters of the string.

          y
            Y-coordinate.
          x
            X-coordinate.
          str
            String to add.
          n
            Maximal number of characters.
          attr
            Attributes for characters.

        Paint at most n characters of the string str at (y, x) with
        attributes attr, overwriting anything previously on the display.
        By default, the character position and attributes are the
        current settings for the window object.
        """
    @overload
    def addnstr(self, y: int, x: int, str: str, n: int, attr: int = ...) -> None: ...

    @overload
    def addstr(self, str: str, attr: int = ...) -> None:
        """addstr([y, x,] str, [attr])
        Paint the string.

          y
            Y-coordinate.
          x
            X-coordinate.
          str
            String to add.
          attr
            Attributes for characters.

        Paint the string str at (y, x) with attributes attr,
        overwriting anything previously on the display.
        By default, the character position and attributes are the
        current settings for the window object.
        """
    @overload
    def addstr(self, y: int, x: int, str: str, attr: int = ...) -> None: ...

    def attroff(self, attr: int, /) -> None:
        """Remove attribute attr from the "background" set."""

    def attron(self, attr: int, /) -> None:
        """Add attribute attr to the "background" set."""

    def attrset(self, attr: int, /) -> None:
        """Set the "background" set of attributes."""

    def bkgd(self, ch: _ChType, attr: int = 0, /) -> None:
        """Set the background property of the window.

        ch
          Background character.
        attr
          Background attributes.
        """

    def bkgdset(self, ch: _ChType, attr: int = 0, /) -> None:
        """Set the window's background.

        ch
          Background character.
        attr
          Background attributes.
        """

    def border(
        self,
        ls: _ChType = ...,
        rs: _ChType = ...,
        ts: _ChType = ...,
        bs: _ChType = ...,
        tl: _ChType = ...,
        tr: _ChType = ...,
        bl: _ChType = ...,
        br: _ChType = ...,
    ) -> None:
        """Draw a border around the edges of the window.

          ls
            Left side.
          rs
            Right side.
          ts
            Top side.
          bs
            Bottom side.
          tl
            Upper-left corner.
          tr
            Upper-right corner.
          bl
            Bottom-left corner.
          br
            Bottom-right corner.

        Each parameter specifies the character to use for a specific part of
        the border.  The characters can be specified as integers or as
        one-character strings.  A 0 value for any parameter will cause the
        default character to be used for that parameter.
        """

    @overload
    def box(self) -> None:
        """box([verch=0, horch=0])
        Draw a border around the edges of the window.

          verch
            Left and right side.
          horch
            Top and bottom side.

        Similar to border(), but both ls and rs are verch and both ts and bs
        are horch.  The default corner characters are always used by this
        function.
        """
    @overload
    def box(self, vertch: _ChType = 0, horch: _ChType = 0) -> None: ...

    @overload
    def chgat(self, attr: int) -> None:
        """chgat([y, x,] [n=-1,] attr)
        Set the attributes of characters.

          y
            Y-coordinate.
          x
            X-coordinate.
          n
            Number of characters.
          attr
            Attributes for characters.

        Set the attributes of num characters at the current cursor position, or at
        position (y, x) if supplied.  If no value of num is given or num = -1, the
        attribute will be set on all the characters to the end of the line.  This
        function does not move the cursor.  The changed line will be touched using
        the touchline() method so that the contents will be redisplayed by the next
        window refresh.
        """
    @overload
    def chgat(self, num: int, attr: int) -> None: ...
    @overload
    def chgat(self, y: int, x: int, attr: int) -> None: ...
    @overload
    def chgat(self, y: int, x: int, num: int, attr: int) -> None: ...

    def clear(self) -> None:
        """Clear the window and repaint it completely on the next refresh()."""

    def clearok(self, flag: bool, /) -> None:
        """Clear the window on the next refresh() if flag is true."""

    def clrtobot(self) -> None:
        """Erase from the cursor to the end of the window."""

    def clrtoeol(self) -> None:
        """Erase from the cursor to the end of the line."""

    def cursyncup(self) -> None:
        """Update the cursor position of all ancestor windows to match."""

    @overload
    def delch(self) -> None:
        """delch([y, x])
        Delete the character under the cursor, or at (y, x) if specified.

          y
            Y-coordinate.
          x
            X-coordinate.

        All characters to the right on the same line are shifted one
        position left.
        """
    @overload
    def delch(self, y: int, x: int) -> None: ...

    def deleteln(self) -> None:
        """Delete the line under the cursor; move following lines up by one."""

    @overload
    def derwin(self, begin_y: int, begin_x: int) -> window:
        """derwin([nlines=0, ncols=0,] begin_y, begin_x)
        Create a sub-window (window-relative coordinates).

          nlines
            Height.
          ncols
            Width.
          begin_y
            Top side y-coordinate.
          begin_x
            Left side x-coordinate.

        derwin() is the same as calling subwin(), except that begin_y and
        begin_x are relative to the origin of the window, rather than
        relative to the entire screen.
        """
    @overload
    def derwin(self, nlines: int, ncols: int, begin_y: int, begin_x: int) -> window: ...

    def echochar(self, ch: _ChType, attr: int = 0, /) -> None:
        """Add character ch with attribute attr, and refresh.

        ch
          Character to add.
        attr
          Attributes for the character.
        """

    def enclose(self, y: int, x: int, /) -> bool:
        """Return True if the screen-relative coordinates are enclosed by the window.

        y
          Y-coordinate.
        x
          X-coordinate.
        """

    def erase(self) -> None:
        """Clear the window."""

    def getbegyx(self) -> tuple[int, int]:
        """Return a tuple (y, x) of the upper-left corner coordinates."""

    def getbkgd(self) -> tuple[int, int]:
        """Return the window's current background character/attribute pair."""

    @overload
    def getch(self) -> int:
        """getch([y, x])
        Get a character code from terminal keyboard.

          y
            Y-coordinate.
          x
            X-coordinate.

        The integer returned does not have to be in ASCII range: function
        keys, keypad keys and so on return numbers higher than 256.  In
        no-delay mode, -1 is returned if there is no input, else getch()
        waits until a key is pressed.
        """
    @overload
    def getch(self, y: int, x: int) -> int: ...

    @overload
    def get_wch(self) -> int | str:
        """get_wch([y, x])
        Get a wide character from terminal keyboard.

          y
            Y-coordinate.
          x
            X-coordinate.

        Return a character for most keys, or an integer for function keys,
        keypad keys, and other special keys.
        """
    @overload
    def get_wch(self, y: int, x: int) -> int | str: ...

    @overload
    def getkey(self) -> str:
        """getkey([y, x])
        Get a character (string) from terminal keyboard.

          y
            Y-coordinate.
          x
            X-coordinate.

        Returning a string instead of an integer, as getch() does.  Function
        keys, keypad keys and other special keys return a multibyte string
        containing the key name.  In no-delay mode, an exception is raised
        if there is no input.
        """
    @overload
    def getkey(self, y: int, x: int) -> str: ...

    def getmaxyx(self) -> tuple[int, int]:
        """Return a tuple (y, x) of the window height and width."""

    def getparyx(self) -> tuple[int, int]:
        """Return (y, x) relative to the parent window, or (-1, -1) if none."""

    @overload
    def getstr(self) -> bytes:
        """getstr([[y, x,] n=2047])
        Read a string from the user, with primitive line editing capacity.

          y
            Y-coordinate.
          x
            X-coordinate.
          n
            Maximal number of characters.
        """
    @overload
    def getstr(self, n: int) -> bytes: ...
    @overload
    def getstr(self, y: int, x: int) -> bytes: ...
    @overload
    def getstr(self, y: int, x: int, n: int) -> bytes: ...

    def getyx(self) -> tuple[int, int]:
        """Return a tuple (y, x) of the current cursor position."""

    @overload
    def hline(self, ch: _ChType, n: int) -> None:
        """hline([y, x,] ch, n, [attr=_curses.A_NORMAL])
        Display a horizontal line.

          y
            Starting Y-coordinate.
          x
            Starting X-coordinate.
          ch
            Character to draw.
          n
            Line length.
          attr
            Attributes for the characters.
        """
    @overload
    def hline(self, y: int, x: int, ch: _ChType, n: int) -> None: ...

    def idcok(self, flag: bool, /) -> None:
        """Enable or disable the hardware insert/delete character feature."""

    def idlok(self, flag: bool, /) -> None:
        """Enable or disable the hardware insert/delete line feature."""

    def immedok(self, flag: bool, /) -> None:
        """If flag is true, refresh the window on every change to it."""

    @overload
    def inch(self) -> int:
        """inch([y, x])
        Return the character at the given position in the window.

          y
            Y-coordinate.
          x
            X-coordinate.

        The bottom 8 bits are the character proper, and upper bits are the
        attributes.
        """
    @overload
    def inch(self, y: int, x: int) -> int: ...

    @overload
    def insch(self, ch: _ChType, attr: int = ...) -> None:
        """insch([y, x,] ch, [attr=_curses.A_NORMAL])
        Insert a character before the current or specified position.

          y
            Y-coordinate.
          x
            X-coordinate.
          ch
            Character to insert.
          attr
            Attributes for the character.

        All characters to the right of the cursor are shifted one position
        right, with the rightmost characters on the line being lost.
        """
    @overload
    def insch(self, y: int, x: int, ch: _ChType, attr: int = ...) -> None: ...

    def insdelln(self, nlines: int, /) -> None:
        """Insert (nlines > 0) or delete (nlines < 0) lines above the cursor."""

    def insertln(self) -> None:
        """Insert a blank line under the cursor; move following lines down."""

    @overload
    def insnstr(self, str: str, n: int, attr: int = ...) -> None:
        """insnstr([y, x,] str, n, [attr])
        Insert at most n characters of the string.

          y
            Y-coordinate.
          x
            X-coordinate.
          str
            String to insert.
          n
            Maximal number of characters.
          attr
            Attributes for characters.

        Insert a character string (as many characters as will fit on the
        line) before the character under the cursor, up to n characters.  If
        n is zero or negative, the entire string is inserted.  All
        characters to the right of the cursor are shifted right, with the
        rightmost characters on the line being lost.  The cursor position
        does not change (after moving to y, x, if specified).
        """
    @overload
    def insnstr(self, y: int, x: int, str: str, n: int, attr: int = ...) -> None: ...

    @overload
    def insstr(self, str: str, attr: int = ...) -> None:
        """insstr([y, x,] str, [attr])
        Insert the string before the current or specified position.

          y
            Y-coordinate.
          x
            X-coordinate.
          str
            String to insert.
          attr
            Attributes for characters.

        Insert a character string (as many characters as will fit on the
        line) before the character under the cursor.  All characters to the
        right of the cursor are shifted right, with the rightmost characters
        on the line being lost.  The cursor position does not change (after
        moving to y, x, if specified).
        """
    @overload
    def insstr(self, y: int, x: int, str: str, attr: int = ...) -> None: ...

    @overload
    def instr(self, n: int = 2047) -> bytes:
        """instr([y, x,] n=2047)
        Return a string of characters, extracted from the window.

          y
            Y-coordinate.
          x
            X-coordinate.
          n
            Maximal number of characters.

        Return a string of characters, extracted from the window starting
        at the current cursor position, or at y, x if specified, and
        stopping at the end of the line.  Attributes and color
        information are stripped from the characters.  If n is specified,
        instr() returns a string at most n characters long (exclusive of
        the trailing NUL).
        """
    @overload
    def instr(self, y: int, x: int, n: int = 2047) -> bytes: ...

    def is_linetouched(self, line: int, /) -> bool:
        """Return True if the specified line was modified, otherwise return False.

          line
            Line number.

        Raise a curses.error exception if line is not valid for the given
        window.
        """

    def is_wintouched(self) -> bool:
        """Return True if the window changed since the last refresh()."""

    def keypad(self, flag: bool, /) -> None:
        """Interpret escape sequences for special keys if flag is true."""

    def leaveok(self, flag: bool, /) -> None:
        """If flag is true, leave the cursor where the update leaves it."""

    def move(self, new_y: int, new_x: int, /) -> None:
        """Move the cursor to (new_y, new_x)."""

    def mvderwin(self, y: int, x: int, /) -> None:
        """Move the window inside its parent window."""

    def mvwin(self, new_y: int, new_x: int, /) -> None:
        """Move the window so its upper-left corner is at (new_y, new_x)."""

    def nodelay(self, flag: bool, /) -> None:
        """If flag is true, getch() becomes non-blocking."""

    def notimeout(self, flag: bool, /) -> None:
        """If flag is true, do not time out escape sequences."""

    @overload
    def noutrefresh(self) -> None:
        """noutrefresh([pminrow, pmincol, sminrow, smincol, smaxrow, smaxcol])
        Mark for refresh but wait.

        This function updates the data structure representing the desired
        state of the window, but does not force an update of the physical
        screen.  To accomplish that, call doupdate().
        """
    @overload
    def noutrefresh(self, pminrow: int, pmincol: int, sminrow: int, smincol: int, smaxrow: int, smaxcol: int) -> None: ...

    @overload
    def overlay(self, destwin: window) -> None:
        """overlay(destwin, [sminrow, smincol, dminrow, dmincol, dmaxrow, dmaxcol])
        Overlay the window on top of destwin.

        The windows need not be the same size, only the overlapping region
        is copied.  This copy is non-destructive, which means that the
        current background character does not overwrite the old contents of
        destwin.

        To get fine-grained control over the copied region, the second form
        of overlay() can be used.  sminrow and smincol are the upper-left
        coordinates of the source window, and the other variables mark
        a rectangle in the destination window.
        """
    @overload
    def overlay(
        self, destwin: window, sminrow: int, smincol: int, dminrow: int, dmincol: int, dmaxrow: int, dmaxcol: int
    ) -> None: ...

    @overload
    def overwrite(self, destwin: window) -> None:
        """overwrite(destwin, [sminrow, smincol, dminrow, dmincol, dmaxrow,
                  dmaxcol])
        Overwrite the window on top of destwin.

        The windows need not be the same size, in which case only the
        overlapping region is copied.  This copy is destructive, which means
        that the current background character overwrites the old contents of
        destwin.

        To get fine-grained control over the copied region, the second form
        of overwrite() can be used. sminrow and smincol are the upper-left
        coordinates of the source window, the other variables mark
        a rectangle in the destination window.
        """
    @overload
    def overwrite(
        self, destwin: window, sminrow: int, smincol: int, dminrow: int, dmincol: int, dmaxrow: int, dmaxcol: int
    ) -> None: ...

    def putwin(self, file: SupportsWrite[bytes], /) -> None:
        """Write all data associated with the window into the provided file object.

        This information can be later retrieved using the getwin() function.
        """

    def redrawln(self, beg: int, num: int, /) -> None:
        """Mark the specified lines corrupted.

          beg
            Starting line number.
          num
            The number of lines.

        They should be completely redrawn on the next refresh() call.
        """

    def redrawwin(self) -> None:
        """Mark the entire window for redraw on the next refresh()."""

    @overload
    def refresh(self) -> None:
        """refresh([pminrow, pmincol, sminrow, smincol, smaxrow, smaxcol])
        Update the display immediately.

        Synchronize actual screen with previous drawing/deleting methods.
        The 6 optional arguments can only be specified when the window is
        a pad created with newpad().  The additional parameters are needed
        to indicate what part of the pad and screen are involved.  pminrow
        and pmincol specify the upper left-hand corner of the rectangle to
        be displayed in the pad.  sminrow, smincol, smaxrow, and smaxcol
        specify the edges of the rectangle to be displayed on the screen.
        The lower right-hand corner of the rectangle to be displayed in the
        pad is calculated from the screen coordinates, since the rectangles
        must be the same size.  Both rectangles must be entirely contained
        within their respective structures.  Negative values of pminrow,
        pmincol, sminrow, or smincol are treated as if they were zero.
        """
    @overload
    def refresh(self, pminrow: int, pmincol: int, sminrow: int, smincol: int, smaxrow: int, smaxcol: int) -> None: ...

    def resize(self, nlines: int, ncols: int, /) -> None:
        """Resize the window to nlines rows and ncols columns."""

    def scroll(self, lines: int = 1) -> None:
        """scroll([lines=1])
        Scroll the screen or scrolling region.

          lines
            Number of lines to scroll.

        Scroll upward if the argument is positive and downward if it is
        negative.
        """

    def scrollok(self, flag: bool, /) -> None:
        """Control whether the window scrolls when the cursor moves off it."""

    def setscrreg(self, top: int, bottom: int, /) -> None:
        """Define a software scrolling region.

          top
            First line number.
          bottom
            Last line number.

        All scrolling actions will take place in this region.
        """

    def standend(self) -> None:
        """Turn off the standout attribute."""

    def standout(self) -> None:
        """Turn on the A_STANDOUT attribute."""

    @overload
    def subpad(self, begin_y: int, begin_x: int) -> window:
        """subwin([nlines=0, ncols=0,] begin_y, begin_x)
        Create a sub-window (screen-relative coordinates).

          nlines
            Height.
          ncols
            Width.
          begin_y
            Top side y-coordinate.
          begin_x
            Left side x-coordinate.

        By default, the sub-window will extend from the specified position
        to the lower right corner of the window.
        """
    @overload
    def subpad(self, nlines: int, ncols: int, begin_y: int, begin_x: int) -> window: ...

    @overload
    def subwin(self, begin_y: int, begin_x: int) -> window:
        """subwin([nlines=0, ncols=0,] begin_y, begin_x)
        Create a sub-window (screen-relative coordinates).

          nlines
            Height.
          ncols
            Width.
          begin_y
            Top side y-coordinate.
          begin_x
            Left side x-coordinate.

        By default, the sub-window will extend from the specified position
        to the lower right corner of the window.
        """
    @overload
    def subwin(self, nlines: int, ncols: int, begin_y: int, begin_x: int) -> window: ...

    def syncdown(self) -> None:
        """Touch each location changed in any ancestor of the window."""

    def syncok(self, flag: bool, /) -> None:
        """If flag is true, call syncup() on every change to the window."""

    def syncup(self) -> None:
        """Touch locations in ancestors that changed in this window."""

    def timeout(self, delay: int, /) -> None:
        """Set blocking or non-blocking read behavior for the window."""

    def touchline(self, start: int, count: int, changed: bool = True) -> None:
        """touchline(start, count, [changed=True])
        Pretend count lines have been changed, starting with line start.

        If changed is supplied, it specifies whether the affected lines are
        marked as having been changed (changed=True) or unchanged
        (changed=False).
        """

    def touchwin(self) -> None:
        """Mark the whole window as changed."""

    def untouchwin(self) -> None:
        """Mark all lines in the window as unchanged since last refresh()."""

    @overload
    def vline(self, ch: _ChType, n: int) -> None:
        """vline([y, x,] ch, n, [attr=_curses.A_NORMAL])
        Display a vertical line.

          y
            Starting Y-coordinate.
          x
            Starting X-coordinate.
          ch
            Character to draw.
          n
            Line length.
          attr
            Attributes for the character.
        """
    @overload
    def vline(self, y: int, x: int, ch: _ChType, n: int) -> None: ...

ncurses_version: _ncurses_version



"""
Path operations common to more than one OS
Do not use directly.  The OS specific modules import the appropriate
functions from this module themselves.
"""

import os
import sys
from _typeshed import BytesPath, FileDescriptorOrPath, StrOrBytesPath, StrPath, SupportsRichComparisonT
from collections.abc import Sequence
from typing import Literal, NewType, overload
from typing_extensions import LiteralString, deprecated

__all__ = [
    "commonprefix",
    "exists",
    "getatime",
    "getctime",
    "getmtime",
    "getsize",
    "isdir",
    "isfile",
    "samefile",
    "sameopenfile",
    "samestat",
    "ALLOW_MISSING",
]
if sys.version_info >= (3, 12):
    __all__ += ["islink"]
if sys.version_info >= (3, 13):
    __all__ += ["isjunction", "isdevdrive", "lexists"]
if sys.version_info >= (3, 15):
    __all__ += ["ALL_BUT_LAST"]

# All overloads can return empty string. Ideally, Literal[""] would be a valid
# Iterable[T], so that list[T] | Literal[""] could be used as a return
# type. But because this only works when T is str, we need Sequence[T] instead.
if sys.version_info >= (3, 15):
    @overload
    @deprecated("Deprecated since Python 3.15; use os.path.commonpath() for path prefixes.")
    def commonprefix(m: Sequence[LiteralString], /) -> LiteralString:
        """Given a list of pathnames, returns the longest common leading component"""
    @overload
    @deprecated("Deprecated since Python 3.15; use os.path.commonpath() for path prefixes.")
    def commonprefix(m: Sequence[StrPath], /) -> str: ...
    @overload
    @deprecated("Deprecated since Python 3.15; use os.path.commonpath() for path prefixes.")
    def commonprefix(m: Sequence[BytesPath], /) -> bytes | Literal[""]: ...
    @overload
    @deprecated("Deprecated since Python 3.15; use os.path.commonpath() for path prefixes.")
    def commonprefix(m: Sequence[list[SupportsRichComparisonT]], /) -> Sequence[SupportsRichComparisonT]: ...
    @overload
    @deprecated("Deprecated since Python 3.15; use os.path.commonpath() for path prefixes.")
    def commonprefix(m: Sequence[tuple[SupportsRichComparisonT, ...]], /) -> Sequence[SupportsRichComparisonT]: ...
else:
    @overload
    def commonprefix(m: Sequence[LiteralString]) -> LiteralString:
        """Given a list of pathnames, returns the longest common leading component"""
    @overload
    def commonprefix(m: Sequence[StrPath]) -> str: ...
    @overload
    def commonprefix(m: Sequence[BytesPath]) -> bytes | Literal[""]: ...
    @overload
    def commonprefix(m: Sequence[list[SupportsRichComparisonT]]) -> Sequence[SupportsRichComparisonT]: ...
    @overload
    def commonprefix(m: Sequence[tuple[SupportsRichComparisonT, ...]]) -> Sequence[SupportsRichComparisonT]: ...

def exists(path: FileDescriptorOrPath) -> bool:
    """Test whether a path exists.  Returns False for broken symbolic links"""

def isfile(path: FileDescriptorOrPath) -> bool:
    """Test whether a path is a regular file"""

def isdir(s: FileDescriptorOrPath) -> bool:
    """Return true if the pathname refers to an existing directory."""

if sys.version_info >= (3, 12):
    def islink(path: StrOrBytesPath) -> bool:
        """Test whether a path is a symbolic link"""

# These return float if os.stat_float_times() == True,
# but int is a subclass of float.
def sameopenfile(fp1: int, fp2: int) -> bool:
    """Test whether two open file objects reference the same file"""

if sys.version_info >= (3, 15):
    def getsize(filename: FileDescriptorOrPath, /) -> int:
        """Return the size of a file, reported by os.stat()."""

    def getatime(filename: FileDescriptorOrPath, /) -> float:
        """Return the last access time of a file, reported by os.stat()."""

    def getmtime(filename: FileDescriptorOrPath, /) -> float:
        """Return the last modification time of a file, reported by os.stat()."""

    def getctime(filename: FileDescriptorOrPath, /) -> float:
        """Return the metadata change time of a file, reported by os.stat()."""

    def samefile(f1: FileDescriptorOrPath, f2: FileDescriptorOrPath, /) -> bool:
        """Test whether two pathnames reference the same actual file or directory

        This is determined by the device number and i-node number and
        raises an exception if an os.stat() call on either pathname fails.
        """

    def samestat(s1: os.stat_result, s2: os.stat_result, /) -> bool:
        """Test whether two stat buffers reference the same file"""

else:
    def getsize(filename: FileDescriptorOrPath) -> int:
        """Return the size of a file, reported by os.stat()."""

    def getatime(filename: FileDescriptorOrPath) -> float:
        """Return the last access time of a file, reported by os.stat()."""

    def getmtime(filename: FileDescriptorOrPath) -> float:
        """Return the last modification time of a file, reported by os.stat()."""

    def getctime(filename: FileDescriptorOrPath) -> float:
        """Return the metadata change time of a file, reported by os.stat()."""

    def samefile(f1: FileDescriptorOrPath, f2: FileDescriptorOrPath) -> bool:
        """Test whether two pathnames reference the same actual file or directory

        This is determined by the device number and i-node number and
        raises an exception if an os.stat() call on either pathname fails.
        """

    def samestat(s1: os.stat_result, s2: os.stat_result) -> bool:
        """Test whether two stat buffers reference the same file"""

if sys.version_info >= (3, 13):
    def isjunction(path: StrOrBytesPath) -> bool:
        """Test whether a path is a junction
        Junctions are not supported on the current platform
        """

    def isdevdrive(path: StrOrBytesPath) -> bool:
        """Determines whether the specified path is on a Windows Dev Drive.
        Dev Drives are not supported on the current platform
        """

    def lexists(path: StrOrBytesPath) -> bool:
        """Test whether a path exists.  Returns True for broken symbolic links"""

# Added in Python 3.9.23, 3.10.18, 3.11.13, 3.12.11, 3.13.4
_AllowMissingType = NewType("_AllowMissingType", object)
ALLOW_MISSING: _AllowMissingType

if sys.version_info >= (3, 15):
    _AllButLastType = NewType("_AllButLastType", object)
    ALL_BUT_LAST: _AllButLastType



# Regression tests for https://github.com/astral-sh/ruff/issues/11569


# comment 1
def foo(self) -> None: ...
def bar(self) -> None: ...
# comment 2

# comment 3
def baz(self) -> None:
    return None
# comment 4


def foo(self) -> None: ...
# comment 5

def baz(self) -> None:
    return None


def foo(self) -> None:
    ... # comment 5
def baz(self) -> None:
    return None

def foo(self) -> None: ...
# comment 5



"""Support to pretty-print lists, tuples, & dictionaries recursively.

Very simple, but useful, especially in debugging data structures.

Classes
-------

PrettyPrinter()
    Handle pretty-printing operations onto a stream using a configured
    set of formatting parameters.

Functions
---------

pformat()
    Format a Python object into a pretty-printed representation.

pprint()
    Pretty-print a Python object to a stream [default is sys.stdout].

saferepr()
    Generate a 'standard' repr()-like value, but protect against recursive
    data structures.

"""

import sys
from _typeshed import SupportsWrite
from collections import deque
from typing import IO

__all__ = ["pprint", "pformat", "isreadable", "isrecursive", "saferepr", "PrettyPrinter", "pp"]

if sys.version_info >= (3, 15):
    # The `expand` parameter was added in Python 3.15.
    def pformat(
        object: object,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        expand: bool = False,
        sort_dicts: bool = True,
        underscore_numbers: bool = False,
    ) -> str:
        """Format a Python object into a pretty-printed representation."""

else:
    def pformat(
        object: object,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        sort_dicts: bool = True,
        underscore_numbers: bool = False,
    ) -> str:
        """Format a Python object into a pretty-printed representation."""

if sys.version_info >= (3, 15):
    # The `expand` parameter was added in Python 3.15.
    def pp(
        object: object,
        stream: IO[str] | None = None,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        expand: bool = False,
        sort_dicts: bool = False,
        underscore_numbers: bool = False,
    ) -> None:
        """Pretty-print a Python object"""

else:
    def pp(
        object: object,
        stream: IO[str] | None = None,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        sort_dicts: bool = False,
        underscore_numbers: bool = False,
    ) -> None:
        """Pretty-print a Python object"""

if sys.version_info >= (3, 15):
    # The `expand` parameter was added in Python 3.15.
    def pprint(
        object: object,
        stream: IO[str] | None = None,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        expand: bool = False,
        sort_dicts: bool = True,
        underscore_numbers: bool = False,
    ) -> None:
        """Pretty-print a Python object to a stream [default is sys.stdout]."""

else:
    def pprint(
        object: object,
        stream: IO[str] | None = None,
        indent: int = 1,
        width: int = 80,
        depth: int | None = None,
        *,
        compact: bool = False,
        sort_dicts: bool = True,
        underscore_numbers: bool = False,
    ) -> None:
        """Pretty-print a Python object to a stream [default is sys.stdout]."""

def isreadable(object: object) -> bool:
    """Determine if saferepr(object) is readable by eval()."""

def isrecursive(object: object) -> bool:
    """Determine if object requires a recursive representation."""

def saferepr(object: object) -> str:
    """Version of repr() which can handle recursive data structures."""

class PrettyPrinter:
    if sys.version_info >= (3, 15):
        # The `expand` parameter was added in Python 3.15.
        def __init__(
            self,
            indent: int = 1,
            width: int = 80,
            depth: int | None = None,
            stream: IO[str] | None = None,
            *,
            compact: bool = False,
            expand: bool = False,
            sort_dicts: bool = True,
            underscore_numbers: bool = False,
        ) -> None:
            """Handle pretty printing operations onto a stream using a set of
            configured parameters.

            indent
                Number of spaces to indent for each level of nesting.

            width
                Attempted maximum number of columns in the output.

            depth
                The maximum depth to print out nested structures.

            stream
                The desired output stream.  If omitted (or false), the standard
                output stream available at construction will be used.

            compact
                If true, several items will be combined in one line.
                Incompatible with expand mode.

            expand
                If true, the output will be formatted similar to
                pretty-printed json.dumps() when ``indent`` is supplied.
                Incompatible with compact mode.

            sort_dicts
                If true, dict keys are sorted.

            underscore_numbers
                If true, digit groups are separated with underscores.

            """

    else:
        def __init__(
            self,
            indent: int = 1,
            width: int = 80,
            depth: int | None = None,
            stream: IO[str] | None = None,
            *,
            compact: bool = False,
            sort_dicts: bool = True,
            underscore_numbers: bool = False,
        ) -> None:
            """Handle pretty printing operations onto a stream using a set of
            configured parameters.

            indent
                Number of spaces to indent for each level of nesting.

            width
                Attempted maximum number of columns in the output.

            depth
                The maximum depth to print out nested structures.

            stream
                The desired output stream.  If omitted (or false), the standard
                output stream available at construction will be used.

            compact
                If true, several items will be combined in one line.

            sort_dicts
                If true, dict keys are sorted.

            underscore_numbers
                If true, digit groups are separated with underscores.

            """

    def pformat(self, object: object) -> str: ...
    def pprint(self, object: object) -> None: ...
    def isreadable(self, object: object) -> bool: ...
    def isrecursive(self, object: object) -> bool: ...
    def format(self, object: object, context: dict[int, int], maxlevels: int, level: int) -> tuple[str, bool, bool]:
        """Format object for a specific context, returning a string
        and flags indicating whether the representation is 'readable'
        and whether the object represents a recursive construct.
        """

    def _format(
        self, object: object, stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _pprint_dict(
        self,
        object: dict[object, object],
        stream: SupportsWrite[str],
        indent: int,
        allowance: int,
        context: dict[int, int],
        level: int,
    ) -> None: ...
    def _pprint_list(
        self, object: list[object], stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _pprint_tuple(
        self,
        object: tuple[object, ...],
        stream: SupportsWrite[str],
        indent: int,
        allowance: int,
        context: dict[int, int],
        level: int,
    ) -> None: ...
    def _pprint_set(
        self, object: set[object], stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _pprint_deque(
        self, object: deque[object], stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _format_dict_items(
        self,
        items: list[tuple[object, object]],
        stream: SupportsWrite[str],
        indent: int,
        allowance: int,
        context: dict[int, int],
        level: int,
    ) -> None: ...
    def _format_items(
        self, items: list[object], stream: SupportsWrite[str], indent: int, allowance: int, context: dict[int, int], level: int
    ) -> None: ...
    def _repr(self, object: object, context: dict[int, int], level: int) -> str: ...
    def _safe_repr(self, object: object, context: dict[int, int], maxlevels: int, level: int) -> tuple[str, bool, bool]: ...



import ssl
import sys
from _typeshed import ReadableBuffer, StrPath
from collections.abc import Awaitable, Callable, Iterable, Sequence, Sized
from types import ModuleType
from typing import Any, Protocol, SupportsIndex, TypeAlias, type_check_only
from typing_extensions import Self

from . import events, protocols, transports
from .base_events import Server

# Keep asyncio.__all__ updated with any changes to __all__ here
if sys.platform == "win32":
    __all__ = ("StreamReader", "StreamWriter", "StreamReaderProtocol", "open_connection", "start_server")
else:
    __all__ = (
        "StreamReader",
        "StreamWriter",
        "StreamReaderProtocol",
        "open_connection",
        "start_server",
        "open_unix_connection",
        "start_unix_server",
    )

_ClientConnectedCallback: TypeAlias = Callable[[StreamReader, StreamWriter], Awaitable[None] | None]

@type_check_only
class _ReaduntilBuffer(ReadableBuffer, Sized, Protocol): ...

async def open_connection(
    host: str | None = None,
    port: int | str | None = None,
    *,
    limit: int = 65536,
    ssl_handshake_timeout: float | None = None,
    **kwds: Any,
) -> tuple[StreamReader, StreamWriter]:
    """A wrapper for create_connection() returning a (reader, writer) pair.

    The reader returned is a StreamReader instance; the writer is a
    StreamWriter instance.

    The arguments are all the usual arguments to create_connection()
    except protocol_factory; most common are positional host and port,
    with various optional keyword arguments following.

    Additional optional keyword arguments are loop (to set the event loop
    instance to use) and limit (to set the buffer limit passed to the
    StreamReader).

    (If you want to customize the StreamReader and/or
    StreamReaderProtocol classes, just copy the code -- there's
    really nothing special here except some convenience.)
    """

async def start_server(
    client_connected_cb: _ClientConnectedCallback,
    host: str | Sequence[str] | None = None,
    port: int | str | None = None,
    *,
    limit: int = 65536,
    ssl_handshake_timeout: float | None = None,
    **kwds: Any,
) -> Server:
    """Start a socket server, call back for each client connected.

    The first parameter, `client_connected_cb`, takes two parameters:
    client_reader, client_writer.  client_reader is a StreamReader
    object, while client_writer is a StreamWriter object.  This
    parameter can either be a plain callback function or a coroutine;
    if it is a coroutine, it will be automatically converted into a
    Task.

    The rest of the arguments are all the usual arguments to
    loop.create_server() except protocol_factory; most common are
    positional host and port, with various optional keyword arguments
    following.  The return value is the same as loop.create_server().

    Additional optional keyword argument is limit (to set the buffer
    limit passed to the StreamReader).

    The return value is the same as loop.create_server(), i.e. a
    Server object which can be used to stop the service.
    """

if sys.platform != "win32":
    async def open_unix_connection(
        path: StrPath | None = None, *, limit: int = 65536, **kwds: Any
    ) -> tuple[StreamReader, StreamWriter]:
        """Similar to `open_connection` but works with UNIX Domain Sockets."""

    async def start_unix_server(
        client_connected_cb: _ClientConnectedCallback, path: StrPath | None = None, *, limit: int = 65536, **kwds: Any
    ) -> Server:
        """Similar to `start_server` but works with UNIX Domain Sockets."""

class FlowControlMixin(protocols.Protocol):
    """Reusable flow control logic for StreamWriter.drain().

    This implements the protocol methods pause_writing(),
    resume_writing() and connection_lost().  If the subclass overrides
    these it must call the super methods.

    StreamWriter.drain() must wait for _drain_helper() coroutine.
    """

    def __init__(self, loop: events.AbstractEventLoop | None = None) -> None: ...

class StreamReaderProtocol(FlowControlMixin, protocols.Protocol):
    """Helper class to adapt between Protocol and StreamReader.

    (This is a helper class instead of making StreamReader itself a
    Protocol subclass, because the StreamReader has other potential
    uses, and to prevent the user of the StreamReader to accidentally
    call inappropriate methods of the protocol.)
    """

    def __init__(
        self,
        stream_reader: StreamReader,
        client_connected_cb: _ClientConnectedCallback | None = None,
        loop: events.AbstractEventLoop | None = None,
    ) -> None: ...
    def __del__(self) -> None: ...

class StreamWriter:
    """Wraps a Transport.

    This exposes write(), writelines(), [can_]write_eof(),
    get_extra_info() and close().  It adds drain() which returns an
    optional Future on which you can wait for flow control.  It also
    adds a transport property which references the Transport
    directly.
    """

    def __init__(
        self,
        transport: transports.WriteTransport,
        protocol: protocols.BaseProtocol,
        reader: StreamReader | None,
        loop: events.AbstractEventLoop,
    ) -> None: ...
    @property
    def transport(self) -> transports.WriteTransport: ...
    def write(self, data: bytes | bytearray | memoryview) -> None: ...
    def writelines(self, data: Iterable[bytes | bytearray | memoryview]) -> None: ...
    def write_eof(self) -> None: ...
    def can_write_eof(self) -> bool: ...
    def close(self) -> None: ...
    def is_closing(self) -> bool: ...
    async def wait_closed(self) -> None: ...
    def get_extra_info(self, name: str, default: Any = None) -> Any: ...
    async def drain(self) -> None:
        """Flush the write buffer.

        The intended use is to write

          w.write(data)
          await w.drain()
        """

    if sys.version_info >= (3, 12):
        async def start_tls(
            self,
            sslcontext: ssl.SSLContext,
            *,
            server_hostname: str | None = None,
            ssl_handshake_timeout: float | None = None,
            ssl_shutdown_timeout: float | None = None,
        ) -> None:
            """Upgrade an existing stream-based connection to TLS."""

    elif sys.version_info >= (3, 11):
        async def start_tls(
            self, sslcontext: ssl.SSLContext, *, server_hostname: str | None = None, ssl_handshake_timeout: float | None = None
        ) -> None:
            """Upgrade an existing stream-based connection to TLS."""

    if sys.version_info >= (3, 13):
        def __del__(self, warnings: ModuleType = ...) -> None: ...
    elif sys.version_info >= (3, 11):
        def __del__(self) -> None: ...

class StreamReader:
    def __init__(self, limit: int = 65536, loop: events.AbstractEventLoop | None = None) -> None: ...
    def exception(self) -> Exception | None: ...
    def set_exception(self, exc: Exception) -> None: ...
    def set_transport(self, transport: transports.BaseTransport) -> None: ...
    def feed_eof(self) -> None: ...
    def at_eof(self) -> bool:
        """Return True if the buffer is empty and 'feed_eof' was called."""

    def feed_data(self, data: Iterable[SupportsIndex]) -> None: ...
    async def readline(self) -> bytes:
        """Read chunk of data from the stream until newline (b'\\n') is found.

        On success, return chunk that ends with newline.  If only partial
        line can be read due to EOF, return incomplete line without
        terminating newline.  When EOF was reached while no bytes read,
        empty bytes object is returned.

        If limit is reached, ValueError will be raised.  In that case, if
        newline was found, complete line including newline will be removed
        from internal buffer.  Else, internal buffer will be cleared.
        Limit is compared against part of the line without newline.

        If stream was paused, this function will automatically resume it if
        needed.
        """

    if sys.version_info >= (3, 13):
        async def readuntil(self, separator: _ReaduntilBuffer | tuple[_ReaduntilBuffer, ...] = b"\n") -> bytes:
            """Read data from the stream until ``separator`` is found.

            On success, the data and separator will be removed from the
            internal buffer (consumed). Returned data will include the
            separator at the end.

            Configured stream limit is used to check result. Limit sets the
            maximal length of data that can be returned, not counting the
            separator.

            If an EOF occurs and the complete separator is still not found,
            an IncompleteReadError exception will be raised, and the internal
            buffer will be reset.  The IncompleteReadError.partial attribute
            may contain the separator partially.

            If the data cannot be read because of over limit, a
            LimitOverrunError exception  will be raised, and the data
            will be left in the internal buffer, so it can be read again.

            The ``separator`` may also be a tuple of separators. In this
            case the return value will be the shortest possible that has any
            separator as the suffix. For the purposes of LimitOverrunError,
            the shortest possible separator is considered to be the one that
            matched.
            """

    else:
        async def readuntil(self, separator: _ReaduntilBuffer = b"\n") -> bytes:
            """Read data from the stream until ``separator`` is found.

            On success, the data and separator will be removed from the
            internal buffer (consumed). Returned data will include the
            separator at the end.

            Configured stream limit is used to check result. Limit sets the
            maximal length of data that can be returned, not counting the
            separator.

            If an EOF occurs and the complete separator is still not found,
            an IncompleteReadError exception will be raised, and the internal
            buffer will be reset.  The IncompleteReadError.partial attribute
            may contain the separator partially.

            If the data cannot be read because of over limit, a
            LimitOverrunError exception  will be raised, and the data
            will be left in the internal buffer, so it can be read again.
            """

    async def read(self, n: int = -1) -> bytes:
        """Read up to `n` bytes from the stream.

        If `n` is not provided or set to -1,
        read until EOF, then return all read bytes.
        If EOF was received and the internal buffer is empty,
        return an empty bytes object.

        If `n` is 0, return an empty bytes object immediately.

        If `n` is positive, return at most `n` available bytes
        as soon as at least 1 byte is available in the internal buffer.
        If EOF is received before any byte is read, return an empty
        bytes object.

        Returned value is not limited with limit, configured at stream
        creation.

        If stream was paused, this function will automatically resume it if
        needed.
        """

    async def readexactly(self, n: int) -> bytes:
        """Read exactly `n` bytes.

        Raise an IncompleteReadError if EOF is reached before `n` bytes can be
        read. The IncompleteReadError.partial attribute of the exception will
        contain the partial read bytes.

        if n is zero, return empty bytes object.

        Returned value is not limited with limit, configured at stream
        creation.

        If stream was paused, this function will automatically resume it if
        needed.
        """

    def __aiter__(self) -> Self: ...
    async def __anext__(self) -> bytes: ...



# Leading
lambda x: x  # Trailing
# Trailing

# Leading
lambda x, y: x  # Trailing
# Trailing

# Leading
lambda x, y: x, y  # Trailing
# Trailing

# Leading
lambda x, /, y: x  # Trailing
# Trailing

# Leading
lambda x: lambda y: lambda z: x  # Trailing
# Trailing

# Leading
lambda x: lambda y: lambda z: (x, y, z)  # Trailing
# Trailing

# Leading
lambda x: lambda y: lambda z: (
    x,
    y,
z)  # Trailing
# Trailing

# Leading
lambda x: lambda y: lambda z: (
    x,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
z)  # Trailing
# Trailing

a = (
    lambda  # Dangling
           : 1
)

a = (
    lambda x # Dangling
    , y: 1
)

# Regression test: lambda empty arguments ranges were too long, leading to unstable
# formatting
(lambda:(#
),)

# lambda arguments don't have parentheses, so we never add a magic trailing comma ...
def f(
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa: bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb = lambda x: y,
):
    pass

# ...but we do preserve a trailing comma after the arguments
a = lambda b,: 0

lambda a,: 0
lambda *args,: 0
lambda **kwds,: 0
lambda a, *args,: 0
lambda a, **kwds,: 0
lambda *args, b,: 0
lambda *, b,: 0
lambda *args, **kwds,: 0
lambda a, *args, b,: 0
lambda a, *, b,: 0
lambda a, *args, **kwds,: 0
lambda *args, b, **kwds,: 0
lambda *, b, **kwds,: 0
lambda a, *args, b, **kwds,: 0
lambda a, *, b, **kwds,: 0
lambda a, /: a
lambda a, /, c: a

# Dangling comments without parameters.
(
    lambda
    : # 3
     None
)

(
    lambda
    # 3
    : None
)

(
    lambda  # 1
    # 2
    : # 3
    # 4
    None # 5
)

(
    lambda
    # comment
    *x: x
)

(
    lambda
    # comment
    *x,
    **y: x
)

(
    lambda
    # comment 1
    *
    # comment 2
    x:
    # comment 3
    x
)

(
    lambda # comment 1
    * # comment 2
    x: # comment 3
    x
)

lambda *x\
    :x

(
    lambda
    # comment
    *\
        x: x
)

lambda: ( # comment
    x)

(
    lambda:  # comment
    x
)

(
    lambda:
    # comment
    x
)

(
    lambda  # comment
    :
    x
)

(
    lambda
    # comment
    :
    x
)

(
    lambda:  # comment
    (  # comment
        x
    )
)

(
    lambda  # 1
    # 2
    x  # 3
    # 4
    :  # 5
    # 6
    x
)

(
    lambda  # 1
    # 2
    x,  # 3
    # 4
    y
    :  # 5
    # 6
    x
)

(
    lambda
    x,
    # comment
    y:
    z
)


# Leading
lambda x: (
    lambda y: lambda z: x
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + y
                        + z  # Trailing
)  # Trailing


# Leading
lambda x: lambda y: lambda z: [
    x,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    y,
    z
] # Trailing
# Trailing

lambda self, araa, kkkwargs=aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*args, **kwargs), e=1, f=2, g=2: d

# Regression tests for https://github.com/astral-sh/ruff/issues/8179
def a():
    return b(
        c,
        d,
        e,
        f=lambda self, *args, **kwargs: aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(
            *args, **kwargs
        ),
    )

def a():
    return b(
        c,
        d,
        e,
        f=lambda self, araa, kkkwargs,aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa,
                 args,kwargs,
                 e=1, f=2, g=2: d,
        g = 10
    )

def a():
    return b(
        c,
        d,
        e,
        f=lambda self, *args, **kwargs: aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(
            *args, **kwargs
        ) + 1,
    )

# Additional ecosystem cases from https://github.com/astral-sh/ruff/pull/21385
class C:
    def foo():
        mock_service.return_value.bucket.side_effect = lambda name: (
            source_bucket
            if name == source_bucket_name
            else storage.Bucket(mock_service, destination_bucket_name)
        )

class C:
	function_dict: Dict[Text, Callable[[CRFToken], Any]] = {
        CRFEntityExtractorOptions.POS2: lambda crf_token: crf_token.pos_tag[:2]
        if crf_token.pos_tag is not None
        else None,
	}

name = re.sub(r"[^\x21\x23-\x5b\x5d-\x7e]...............", lambda m: f"\\{m.group(0)}", p["name"])

def foo():
    if True:
        if True:
            return (
                lambda x: np.exp(cs(np.log(x.to(u.MeV).value))) * u.MeV * u.cm**2 / u.g
            )

class C:
    _is_recognized_dtype: Callable[[DtypeObj], bool] = lambda x: lib.is_np_dtype(
        x, "M"
    ) or isinstance(x, DatetimeTZDtype)

class C:
    def foo():
        if True:
            transaction_count = self._query_txs_for_range(
                get_count_fn=lambda from_ts, to_ts, _chain_id=chain_id: db_evmtx.count_transactions_in_range(
                    chain_id=_chain_id,
                    from_ts=from_ts,
                    to_ts=to_ts,
                ),
            )

            transaction_count = self._query_txs_for_range(
                get_count_fn=lambda from_ts, to_ts, _chain_id=chain_id: db_evmtx.count_transactions_in_range[_chain_id, from_ts, to_ts],
            )

def ddb():
    sql = (
        lambda var, table, n=N: f"""
        CREATE TABLE {table} AS
        SELECT ROW_NUMBER() OVER () AS id, {var}
        FROM (
            SELECT {var}
            FROM RANGE({n}) _ ({var})
            ORDER BY RANDOM()
        )
        """
    )

long_assignment_target.with_attribute.and_a_slice[with_an_index] = ( # 1
    # 2
    lambda x, y, z: # 3
    # 4
    x + y + z # 5
    # 6
)

long_assignment_target.with_attribute.and_a_slice[with_an_index] = (
    lambda x, y, z: x + y + z
)

long_assignment_target.with_attribute.and_a_slice[with_an_index] = lambda x, y, z: x + y + z

very_long_variable_name_x, very_long_variable_name_y = lambda a: a + some_very_long_expression, lambda b: b * another_very_long_expression_here

very_long_variable_name_for_result += lambda x: very_long_function_call_that_should_definitely_be_parenthesized_now(x, more_args, additional_parameters)


if 1:
    if 2:
        if 3:
            if self.location in EVM_EVMLIKE_LOCATIONS and database is not None:
                exported_dict["notes"] = EVM_ADDRESS_REGEX.sub(
                    repl=lambda matched_address: self._maybe_add_label_with_address(
                        database=database,
                        matched_address=matched_address,
                    ),
                    string=exported_dict["notes"],
                )

class C:
    def f():
        return dict(
            filter(
                lambda intent_response: self.is_retrieval_intent_response(
                    intent_response
                ),
                self.responses.items(),
            )
        )

@pytest.mark.parametrize(
    "op",
    [
        # Not fluent
        param(
            lambda left, right: (
                ibis.timestamp("2017-04-01")
            ),
        ),
        # These four are fluent and fit on one line inside the parenthesized
        # lambda body
        param(
            lambda left, right: (
                ibis.timestamp("2017-04-01").cast(dt.date)
            ),
        ),
        param(
            lambda left, right: (
                ibis.timestamp("2017-04-01").cast(dt.date).between(left, right)
            ),
        ),
        param(lambda left, right: ibis.timestamp("2017-04-01").cast(dt.date)),
        param(lambda left, right: ibis.timestamp("2017-04-01").cast(dt.date).between(left, right)),
        # This is too long on one line in the lambda body and gets wrapped
        # inside the body.
        param(
            lambda left, right: (
                ibis.timestamp("2017-04-01").cast(dt.date).between(left, right).between(left, right)
            ),
        ),
    ],
)
def test_string_temporal_compare_between(con, op, left, right): ...

[
    (
        lambda eval_df, _: MetricValue(
            scores=eval_df["prediction"].tolist(),
            aggregate_results={"prediction_sum": sum(eval_df["prediction"])},
        )
    ),
]

# reuses the list parentheses
lambda xxxxxxxxxxxxxxxxxxxx, yyyyyyyyyyyyyyyyyyyy, zzzzzzzzzzzzzzzzzzzz: [xxxxxxxxxxxxxxxxxxxx, yyyyyyyyyyyyyyyyyyyy, zzzzzzzzzzzzzzzzzzzz]

# adds parentheses around the body
lambda xxxxxxxxxxxxxxxxxxxx, yyyyyyyyyyyyyyyyyyyy, zzzzzzzzzzzzzzzzzzzz: xxxxxxxxxxxxxxxxxxxx + yyyyyyyyyyyyyyyyyyyy + zzzzzzzzzzzzzzzzzzzz

# removes parentheses around the body
lambda xxxxxxxxxxxxxxxxxxxx: (xxxxxxxxxxxxxxxxxxxx + 1)

mapper = lambda x: dict_with_default[np.nan if isinstance(x, float) and np.isnan(x) else x]

lambda x, y, z: (
    x + y + z
)

lambda x, y, z: (
    x + y + z
    # trailing body
)

lambda x, y, z: (
    x + y + z  # trailing eol body
)

lambda x, y, z: (
    x + y + z
) # trailing lambda

lambda x, y, z: (
    # leading body
    x + y + z
)

lambda x, y, z: (  # leading eol body
    x + y + z
)

(
    lambda name:
    source_bucket  # trailing eol comment
    if name == source_bucket_name
    else storage.Bucket(mock_service, destination_bucket_name)
)

(
    lambda name:
    # dangling header comment
    source_bucket
    if name == source_bucket_name
    else storage.Bucket(mock_service, destination_bucket_name)
)

x = (
    lambda name:
    # dangling header comment
    source_bucket
    if name == source_bucket_name
    else storage.Bucket(mock_service, destination_bucket_name)
)

(
    lambda name: # dangling header comment
    (
        source_bucket
        if name == source_bucket_name
        else storage.Bucket(mock_service, destination_bucket_name)
    )
)

(
    lambda from_ts, to_ts, _chain_id=chain_id:  # dangling eol header comment
    db_evmtx.count_transactions_in_range(
        chain_id=_chain_id,
        from_ts=from_ts,
        to_ts=to_ts,
    )
)

(
    lambda from_ts, to_ts, _chain_id=chain_id:
    # dangling header comment before call
    db_evmtx.count_transactions_in_range(
        chain_id=_chain_id,
        from_ts=from_ts,
        to_ts=to_ts,
    )
)

(
    lambda left, right:
    # comment
    ibis.timestamp("2017-04-01").cast(dt.date).between(left, right)
)

(
    lambda left, right:
    ibis.timestamp("2017-04-01")  # comment
    .cast(dt.date)
    .between(left, right)
)

(
    lambda xxxxxxxxxxxxxxxxxxxx, yyyyyyyyyyyyyyyyyyyy:
    # comment
    [xxxxxxxxxxxxxxxxxxxx, yyyyyyyyyyyyyyyyyyyy, zzzzzzzzzzzzzzzzzzzz]
)

(
    lambda x, y:
    # comment
    {
        "key": x,
        "another": y,
    }
)

(
    lambda x, y:
    # comment
    (
        x,
        y,
        z
    )
)

(
    lambda x:
    # comment
    dict_with_default[np.nan if isinstance(x, float) and np.isnan(x) else x]
)

(
    lambda from_ts, to_ts, _chain_id=chain_id:
    db_evmtx.count_transactions_in_range[
        # comment
        _chain_id, from_ts, to_ts
    ]
)

(
    lambda
    # comment
    *args, **kwargs:
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*args, **kwargs) + 1
)

(
    lambda  # comment
    *args, **kwargs:
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*args, **kwargs) + 1
)

(
    lambda  # comment 1
    # comment 2
    *args, **kwargs: # comment 3
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*args, **kwargs) + 1
)

(
    lambda  # comment 1
    *args, **kwargs: # comment 3
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*args, **kwargs) + 1
)

(
    lambda *args, **kwargs:
    # comment 1
    ( # comment 2
        # comment 3
        aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*args, **kwargs) + 1 # comment 4
        # comment 5
    ) # comment 6
)

(
    lambda *brgs, **kwargs:
    # comment 1
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa( # comment 2
        # comment 3
        *brgs, **kwargs) + 1 # comment 4
    # comment 5
)

(
    lambda *crgs, **kwargs: # comment 1
    aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*crgs, **kwargs) + 1
)

(
    lambda *drgs, **kwargs: # comment 1
    (
        aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*drgs, **kwargs) + 1
    )
)

(
    lambda * # comment 1
    ergs, **
    # comment 2
    kwargs # comment 3
    : # comment 4
    (
        aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa(*ergs, **kwargs) + 1
    )
)

(
    lambda  # 1
    # 2
    left,  # 3
    # 4
    right:  # 5
    # 6
    ibis.timestamp("2017-04-01").cast(dt.date).between(left, right)
)

(
    lambda x:  # outer comment 1
    (
        lambda y:  # inner comment 1
        # inner comment 2
        lambda z: (
            # innermost comment
            x + y + z
        )
    )
)

foo(
    lambda from_ts,  # comment prevents collapsing the parameters to one line
    to_ts, _chain_id=chain_id: db_evmtx.count_transactions_in_range(
        chain_id=_chain_id,
        from_ts=from_ts,
        to_ts=to_ts,
    )
)

foo(
    lambda from_ts,  # but still wrap the body if it gets too long
    to_ts,
    _chain_id=chain_id: db_evmtx.count_transactions_in_rangeeeeeeeeeeeeeeeeeeeeeeeeeeeee(
        chain_id=_chain_id,
        from_ts=from_ts,
        to_ts=to_ts,
    )
)

transform = lambda left, right: ibis.timestamp("2017-04-01").cast(dt.date).between(left, right).between(left, right)  # trailing comment

(
    lambda:  # comment
    1
)

(
    lambda  # comment
    :
    1
)

(
    lambda:
    # comment
    1
)

(
    lambda:  # comment 1
    # comment 2
    1
)

(
    lambda  # comment 1
    # comment 2
    :  # comment 3
    # comment 4
    1
)

(
    lambda
    * # comment 2
    x:
    x
)

(
    lambda # comment 1
    * # comment 2
    x:
    x
)

(
    lambda # comment 1
    y,
    * # comment 2
    x:
    x
)

(
    lambda
    # comment
    *x,
    **y: x
)

(
    lambda
    * # comment 2
    x,
    **y:
    x
)

(
    lambda
    ** # comment 1
    x:
    x
)

(
    lambda: # dangling-end-of-line
    # dangling-own-line
    (  # leading-body-end-of-line
        x
    )
)

(
    lambda: # dangling-end-of-line
    (  # leading-body-end-of-line
        x
    )
)

lambda x: (
    x := 1
)

(
    lambda  # dangling header comment
    : (x := 1)
)


# Regression tests for https://github.com/astral-sh/ruff/issues/23851
foo = lambda x: 'hello this is a really long string that then get concatenated ' 'with another string that prints {x!r}'

foo = lambda x: 'hello this is a really long string that then get concatenated ' 'with another string that prints {x!r}'.format(x=x)

foo = lambda x: 'hello this is a really long string that then get concatenated ' 'with another string that prints {x!r}'[x]

foo = lambda x: ('hello this is a really long string that then get concatenated ' and 'with another string that prints {x!r}').foo()

foo = lambda: (result := some_really_long_callable_expression)(extra_argument_one, extra_argument_two)

foo = lambda: (result := some_really_long_subscriptable_expression)[extra_long_index_expression]

foo = lambda: (await some_coroutine)(extra_argument_one, extra_argument_two, extra_argument_three)

foo = lambda: call()(extra_argument_one, extra_argument_two, extra_argument_threeeeeeeee)

foo = lambda: call()(extra_argument_one, extra_argument_twooooooooooooooooooo, extra_argument_threeeeeeeee)

foo = lambda: call(argument_one,)(extra_argument_one, extra_argument_twooooooooooooooooooo, extra_argument_threeeeeeeee)

foo = lambda: call(argument_one, argument_two, argument_threeeeeeeeeeeeeeeeeeeeeeeeeeee)(extra_argument_one, extra_argument_twooooooooooooooooooo, extra_argument_threeeeeeeee)

foo = lambda: callllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllllll(argument_one, argument_two, argument_threeeeeeeeeeeeeeeeeeeeeeeeeeee)(extra_argument_one, extra_argument_twooooooooooooooooooo, extra_argument_threeeeeeeee)

# Regression tests for https://github.com/astral-sh/ruff/issues/24807
def foo():
    subprocess.check_call(f"rm -rf {' '.join(map(lambda object_name: os.path.join(dest_path, object_name), objects_to_remove))}", shell=True)

def bar():
    lambda x=" ".join(
        map(lambda object_name: os.path.join(dest_path, object_name), objects_to_remove)
    ): ...

value = f"prefix {
    ' '.join(map(lambda object_name: os.path.join(dest_path, object_name), objects_to_remove))
} suffix"

# This can also be triggered in nested, interned elements within `BestFitting` elements
def foo():
    subprocess.check_call(f"rm -rf {' '.join(map(lambda object_name: max(candidate_paths, key=lambda candidate: os.path.join(dest_path, object_name, candidate)), objects_to_remove))}", shell=True)

# Regression test for https://github.com/astral-sh/ruff/issues/25397
class CantFormat:
    args: list[int]

    def __str__(self) -> str:
        return (
            "first string"
            f"list of args long={list(map(lambda arg: str(arg), self.args))})"
        )



"""Fixer for unicode.

* Changes unicode to str and unichr to chr.

* If "...\\u..." is not unicode literal change it into "...\\\\u...".

* Change u"..." into "...".

"""

from _typeshed import StrPath
from typing import ClassVar, Literal

from .. import fixer_base
from ..pytree import Node

class FixUnicode(fixer_base.BaseFix):
    BM_compatible: ClassVar[Literal[True]]
    PATTERN: ClassVar[str]
    unicode_literals: bool
    def start_tree(self, tree: Node, filename: StrPath) -> None: ...
    def transform(self, node, results): ...



"""Python wrapper for Expat parser."""

import sys
from _typeshed import ReadableBuffer, SupportsRead
from collections.abc import Callable
from pyexpat import errors as errors, model as model
from typing import Any, Final, TypeAlias, final
from typing_extensions import CapsuleType
from xml.parsers.expat import ExpatError as ExpatError

EXPAT_VERSION: Final[str]  # undocumented
version_info: tuple[int, int, int]  # undocumented
native_encoding: str  # undocumented
features: list[tuple[str, int]]  # undocumented

error = ExpatError
XML_PARAM_ENTITY_PARSING_NEVER: Final = 0
XML_PARAM_ENTITY_PARSING_UNLESS_STANDALONE: Final = 1
XML_PARAM_ENTITY_PARSING_ALWAYS: Final = 2

_Model: TypeAlias = tuple[int, int, str | None, tuple[Any, ...]]

@final
class XMLParserType:
    """XML parser"""

    def Parse(self, data: str | ReadableBuffer, isfinal: bool = False, /) -> int:
        """Parse XML data.

        'isfinal' should be true at end of input.
        """

    def ParseFile(self, file: SupportsRead[bytes], /) -> int:
        """Parse XML data from file-like object."""

    def SetBase(self, base: str, /) -> None:
        """Set the base URL for the parser."""

    def GetBase(self) -> str | None:
        """Return base URL string for the parser."""

    def GetInputContext(self) -> bytes | None:
        """Return the untranslated text of the input that caused the current event.

        If the event was generated by a large amount of text (such as
        a start tag for an element with many attributes), not all of the
        text may be available.
        """

    def ExternalEntityParserCreate(self, context: str | None, encoding: str = ..., /) -> XMLParserType:
        """Create a parser for parsing an external entity based on the information passed to the ExternalEntityRefHandler."""

    def SetParamEntityParsing(self, flag: int, /) -> int:
        """Controls parsing of parameter entities (including the external DTD subset).

        Possible flag values are XML_PARAM_ENTITY_PARSING_NEVER,
        XML_PARAM_ENTITY_PARSING_UNLESS_STANDALONE and
        XML_PARAM_ENTITY_PARSING_ALWAYS. Returns true if setting the flag
        was successful.
        """

    def UseForeignDTD(self, flag: bool = True, /) -> None:
        """Allows the application to provide an artificial external subset if one is not specified as part of the document instance.

        This readily allows the use of a 'default' document type controlled
        by the application, while still getting the advantage of providing
        document type information to the parser.  'flag' defaults to True if
        not provided.
        """

    def GetReparseDeferralEnabled(self) -> bool:
        """Retrieve reparse deferral enabled status; always returns false with Expat <2.6.0."""

    def SetReparseDeferralEnabled(self, enabled: bool, /) -> None:
        """Enable/Disable reparse deferral; enabled by default with Expat >=2.6.0."""

    # Added in Python 3.10.20, 3.11.15, 3.12.3, 3.13.10, 3.14.1
    def SetAllocTrackerActivationThreshold(self, threshold: int, /) -> None:
        """Sets the number of allocated bytes of dynamic memory needed to activate protection against disproportionate use of RAM.

        Parser objects usually have an allocation activation threshold of
        64 MiB, but the actual default value depends on the underlying Expat
        library.
        """

    def SetAllocTrackerMaximumAmplification(self, max_factor: float, /) -> None:
        """Sets the maximum amplification factor between direct input and bytes of dynamic memory allocated.

        The amplification factor is calculated as "allocated / direct" while
        parsing, where "direct" is the number of bytes read from the primary
        document in parsing and "allocated" is the number of bytes of
        dynamic memory allocated in the parser hierarchy.

        The 'max_factor' value must be a non-NaN floating point value
        greater than or equal to 1.0.  Amplification factors greater than
        100.0 can be observed near the start of parsing even with benign
        files in practice.  In particular, the activation threshold should
        be carefully chosen to avoid false positives.

        Parser objects usually have a maximum amplification factor of 100,
        but the actual default value depends on the underlying Expat
        library.
        """

    if sys.version_info >= (3, 13):
        # Added in Python 3.13.4, 3.14.6
        def SetBillionLaughsAttackProtectionActivationThreshold(self, threshold: int, /) -> None:
            """Sets the number of output bytes needed to activate protection against billion laughs attacks.

            The number of output bytes includes amplification from entity
            expansion and reading DTD files.

            Parser objects usually have a protection activation threshold of
            8 MiB, but the actual default value depends on the underlying Expat
            library.

            Activation thresholds below 4 MiB are known to break support for
            DITA 1.3 payload and are hence not recommended.
            """

        def SetBillionLaughsAttackProtectionMaximumAmplification(self, max_factor: float, /) -> None:
            """Sets the maximum tolerated amplification factor for protection against billion laughs attacks.

            The amplification factor is calculated as "(direct + indirect) /
            direct" while parsing, where "direct" is the number of bytes read
            from the primary document in parsing and "indirect" is the number of
            bytes added by expanding entities and reading external DTD files,
            combined.

            The 'max_factor' value must be a non-NaN floating point value
            greater than or equal to 1.0.  Amplification factors greater than
            30,000 can be observed in the middle of parsing even with benign
            files in practice.  In particular, the activation threshold should
            be carefully chosen to avoid false positives.

            Parser objects usually have a maximum amplification factor of 100,
            but the actual default value depends on the underlying Expat
            library.
            """

    @property
    def intern(self) -> dict[str, str]: ...
    buffer_size: int
    buffer_text: bool
    buffer_used: int
    namespace_prefixes: bool  # undocumented
    ordered_attributes: bool
    specified_attributes: bool
    ErrorByteIndex: int
    ErrorCode: int
    ErrorColumnNumber: int
    ErrorLineNumber: int
    CurrentByteIndex: int
    CurrentColumnNumber: int
    CurrentLineNumber: int
    XmlDeclHandler: Callable[[str, str | None, int], Any] | None
    StartDoctypeDeclHandler: Callable[[str, str | None, str | None, bool], Any] | None
    EndDoctypeDeclHandler: Callable[[], Any] | None
    ElementDeclHandler: Callable[[str, _Model], Any] | None
    AttlistDeclHandler: Callable[[str, str, str, str | None, bool], Any] | None
    StartElementHandler: (
        Callable[[str, dict[str, str]], Any]
        | Callable[[str, list[str]], Any]
        | Callable[[str, dict[str, str], list[str]], Any]
        | None
    )
    EndElementHandler: Callable[[str], Any] | None
    ProcessingInstructionHandler: Callable[[str, str], Any] | None
    CharacterDataHandler: Callable[[str], Any] | None
    UnparsedEntityDeclHandler: Callable[[str, str | None, str, str | None, str], Any] | None
    EntityDeclHandler: Callable[[str, bool, str | None, str | None, str, str | None, str | None], Any] | None
    NotationDeclHandler: Callable[[str, str | None, str, str | None], Any] | None
    StartNamespaceDeclHandler: Callable[[str, str], Any] | None
    EndNamespaceDeclHandler: Callable[[str], Any] | None
    CommentHandler: Callable[[str], Any] | None
    StartCdataSectionHandler: Callable[[], Any] | None
    EndCdataSectionHandler: Callable[[], Any] | None
    DefaultHandler: Callable[[str], Any] | None
    DefaultHandlerExpand: Callable[[str], Any] | None
    NotStandaloneHandler: Callable[[], int] | None
    ExternalEntityRefHandler: Callable[[str, str | None, str | None, str | None], int] | None
    SkippedEntityHandler: Callable[[str, bool], Any] | None

def ErrorString(code: int, /) -> str:
    """Returns string error for given number."""

# intern is undocumented
def ParserCreate(
    encoding: str | None = None, namespace_separator: str | None = None, intern: dict[str, Any] | None = None
) -> XMLParserType:
    """Return a new XML parser object."""

expat_CAPI: CapsuleType



def needs_parentheses( ) -> bool:
    return item.sizing_mode is None and <RANGE_START>item.width_policy == "auto" and item.height_policy == "automatic"<RANGE_END>

def no_longer_needs_parentheses( ) -> bool:
    return (
        <RANGE_START>item.width_policy == "auto"
        and item.height_policy == "automatic"<RANGE_END>
    )


def format_range_after_inserted_parens   ():
    a and item.sizing_mode is None and item.width_policy == "auto" and item.height_policy == "automatic"<RANGE_START>
    print("Format this" ) <RANGE_END>



# Comments are normalized by inserting a single leading space after the `#`,
# unless they start with one of a few special characters. Those are preserved
# verbatim so that tooling relying on a specific prefix keeps working.

#! shebang-style comments are left as-is
#: Sphinx-style comments are left as-is
#' pweave-style comments are left as-is
#| Quarto cell options are left as-is



type A[T=int] = float
type B[*P=int] = float
type C[*Ts=int] = float
type D[*Ts=*int] = float
type D[something_that_is_very_very_very_long=something_that_is_very_very_very_long] = float
type D[*something_that_is_very_very_very_long=*something_that_is_very_very_very_long] = float
type something_that_is_long[something_that_is_long=something_that_is_long] = something_that_is_long

def simple[T=something_that_is_long](short1: int, short2: str, short3: bytes) -> float:
    pass

def longer[something_that_is_long=something_that_is_long](something_that_is_long: something_that_is_long) -> something_that_is_long:
    pass

def trailing_comma1[T=int,](a: str):
    pass

def trailing_comma2[T=int](a: str,):
    pass

def weird_syntax[T=lambda: 42, **P=lambda: 43, *Ts=lambda: 44](): pass



"""Module/script to byte-compile all .py files to .pyc files.

When called as a script with arguments, this compiles the directories
given as arguments recursively; the -l option prevents it from
recursing into directories.

Without arguments, it compiles all modules on sys.path, without
recursing into subdirectories.  (Even though it should do so for
packages -- for now, you'll have to deal with packages separately.)

See module py_compile for details of the actual byte-compilation.
"""

from _typeshed import StrPath
from py_compile import PycInvalidationMode
from typing import Any, Protocol, type_check_only

__all__ = ["compile_dir", "compile_file", "compile_path"]

@type_check_only
class _SupportsSearch(Protocol):
    def search(self, string: str, /) -> Any: ...

def compile_dir(
    dir: StrPath,
    maxlevels: int | None = None,
    ddir: StrPath | None = None,
    force: bool = False,
    rx: _SupportsSearch | None = None,
    quiet: int = 0,
    legacy: bool = False,
    optimize: int = -1,
    workers: int = 1,
    invalidation_mode: PycInvalidationMode | None = None,
    *,
    stripdir: StrPath | None = None,
    prependdir: StrPath | None = None,
    limit_sl_dest: StrPath | None = None,
    hardlink_dupes: bool = False,
) -> bool:
    """Byte-compile all modules in the given directory tree.

    Arguments (only dir is required):

    dir:       the directory to byte-compile
    maxlevels: maximum recursion level (default `sys.getrecursionlimit()`)
    ddir:      the directory that will be prepended to the path to the
               file as it is compiled into each byte-code file.
    force:     if True, force compilation, even if timestamps are up-to-date
    quiet:     full output with False or 0, errors only with 1,
               no output with 2
    legacy:    if True, produce legacy pyc paths instead of PEP 3147 paths
    optimize:  int or list of optimization levels or -1 for level of
               the interpreter. Multiple levels leads to multiple compiled
               files each with one optimization level.
    workers:   maximum number of parallel workers
    invalidation_mode: how the up-to-dateness of the pyc will be checked
    stripdir:  part of path to left-strip from source file path
    prependdir: path to prepend to beginning of original file path, applied
               after stripdir
    limit_sl_dest: ignore symlinks if they are pointing outside of
                   the defined path
    hardlink_dupes: hardlink duplicated pyc files
    """

def compile_file(
    fullname: StrPath,
    ddir: StrPath | None = None,
    force: bool = False,
    rx: _SupportsSearch | None = None,
    quiet: int = 0,
    legacy: bool = False,
    optimize: int = -1,
    invalidation_mode: PycInvalidationMode | None = None,
    *,
    stripdir: StrPath | None = None,
    prependdir: StrPath | None = None,
    limit_sl_dest: StrPath | None = None,
    hardlink_dupes: bool = False,
) -> bool:
    """Byte-compile one file.

    Arguments (only fullname is required):

    fullname:  the file to byte-compile
    ddir:      if given, the directory name compiled in to the
               byte-code file.
    force:     if True, force compilation, even if timestamps are up-to-date
    quiet:     full output with False or 0, errors only with 1,
               no output with 2
    legacy:    if True, produce legacy pyc paths instead of PEP 3147 paths
    optimize:  int or list of optimization levels or -1 for level of
               the interpreter. Multiple levels leads to multiple compiled
               files each with one optimization level.
    invalidation_mode: how the up-to-dateness of the pyc will be checked
    stripdir:  part of path to left-strip from source file path
    prependdir: path to prepend to beginning of original file path, applied
               after stripdir
    limit_sl_dest: ignore symlinks if they are pointing outside of
                   the defined path.
    hardlink_dupes: hardlink duplicated pyc files
    """

def compile_path(
    skip_curdir: bool = ...,
    maxlevels: int = 0,
    force: bool = False,
    quiet: int = 0,
    legacy: bool = False,
    optimize: int = -1,
    invalidation_mode: PycInvalidationMode | None = None,
) -> bool:
    """Byte-compile all module on sys.path.

    Arguments (all optional):

    skip_curdir: if true, skip current directory (default True)
    maxlevels:   max recursion level (default 0)
    force: as for compile_dir() (default False)
    quiet: as for compile_dir() (default 0)
    legacy: as for compile_dir() (default False)
    optimize: as for compile_dir() (default -1)
    invalidation_mode: as for compiler_dir()
    """



"""This module provides primitive operations to manage Python interpreters.
The 'interpreters' module provides a more convenient interface.
"""

import sys
from typing import Any, Literal, SupportsIndex, TypeAlias

_UnboundOp: TypeAlias = Literal[1, 2, 3]

class QueueError(RuntimeError):
    """Indicates that a queue-related error happened."""

class QueueNotFoundError(QueueError): ...

def bind(qid: SupportsIndex) -> None:
    """Take a reference to the identified queue.

    The queue is not destroyed until there are no references left.
    """

if sys.version_info >= (3, 15):
    def create(maxsize: SupportsIndex, unboundop: SupportsIndex = -1, fallback: SupportsIndex = -1) -> int:
        """Create a new cross-interpreter queue and return its unique generated ID.

        It is a new reference as though bind() had been called on the queue.
        The caller is responsible for calling destroy() for the new queue
        before the runtime is finalized.
        """

else:
    def create(maxsize: SupportsIndex, fmt: SupportsIndex, unboundop: _UnboundOp) -> int:
        """create(maxsize, unboundop, fallback) -> qid

        Create a new cross-interpreter queue and return its unique generated ID.
        It is a new reference as though bind() had been called on the queue.

        The caller is responsible for calling destroy() for the new queue
        before the runtime is finalized.
        """

def destroy(qid: SupportsIndex) -> None:
    """Clear and destroy the queue.

    Afterward attempts to use the queue will behave as though it never
    existed.
    """

def get(qid: SupportsIndex) -> tuple[Any, int, _UnboundOp | None]:
    """Return the (object, unbound op) from the front of the queue.

    If there is nothing to receive then raise QueueEmpty.
    """

def get_count(qid: SupportsIndex) -> int:
    """Return the number of items in the queue."""

def get_maxsize(qid: SupportsIndex) -> int:
    """Return the maximum number of items in the queue."""

def get_queue_defaults(qid: SupportsIndex) -> tuple[int, _UnboundOp]:
    """Return the queue's default values, set when it was created."""

def is_full(qid: SupportsIndex) -> bool:
    """Return true if the queue has a maxsize and has reached it."""

def list_all() -> list[tuple[int, int, _UnboundOp]]:
    """Return the list of ID triples for all queues.

    Each ID triple consists of (ID, default unbound op, default fallback).
    """

if sys.version_info >= (3, 15):
    def put(qid: SupportsIndex, obj: Any, unboundop: SupportsIndex = -1, fallback: SupportsIndex = -1) -> None:
        """Add the object's data to the queue."""

else:
    def put(qid: SupportsIndex, obj: Any, fmt: SupportsIndex, unboundop: _UnboundOp) -> None:
        """put(qid, obj)

        Add the object's data to the queue.
        """

def release(qid: SupportsIndex) -> None:
    """Release a reference to the queue.

    The queue is destroyed once there are no references left.
    """



"""Implementation of JSONDecoder"""

import sys
from collections.abc import Callable
from typing import Any

__all__ = ["JSONDecoder", "JSONDecodeError"]

class JSONDecodeError(ValueError):
    """Subclass of ValueError with the following additional properties:

    msg: The unformatted error message
    doc: The JSON document being parsed
    pos: The start index of doc where parsing failed
    lineno: The line corresponding to pos
    colno: The column corresponding to pos

    """

    msg: str
    doc: str
    pos: int
    lineno: int
    colno: int
    def __init__(self, msg: str, doc: str, pos: int) -> None: ...

class JSONDecoder:
    """Simple JSON <https://json.org> decoder

    Performs the following translations in decoding by default:

    +---------------+-------------------+
    | JSON          | Python            |
    +===============+===================+
    | object        | dict              |
    +---------------+-------------------+
    | array         | list              |
    +---------------+-------------------+
    | string        | str               |
    +---------------+-------------------+
    | number (int)  | int               |
    +---------------+-------------------+
    | number (real) | float             |
    +---------------+-------------------+
    | true          | True              |
    +---------------+-------------------+
    | false         | False             |
    +---------------+-------------------+
    | null          | None              |
    +---------------+-------------------+

    It also understands ``NaN``, ``Infinity``, and ``-Infinity`` as
    their corresponding ``float`` values, which is outside the JSON spec.

    """

    if sys.version_info >= (3, 15):
        array_hook: Callable[[list[Any]], Any] | None
    object_hook: Callable[[dict[str, Any]], Any]
    parse_float: Callable[[str], Any]
    parse_int: Callable[[str], Any]
    parse_constant: Callable[[str], Any]
    strict: bool
    object_pairs_hook: Callable[[list[tuple[str, Any]]], Any]
    if sys.version_info >= (3, 15):
        def __init__(
            self,
            *,
            object_hook: Callable[[dict[str, Any]], Any] | None = None,
            parse_float: Callable[[str], Any] | None = None,
            parse_int: Callable[[str], Any] | None = None,
            parse_constant: Callable[[str], Any] | None = None,
            strict: bool = True,
            object_pairs_hook: Callable[[list[tuple[str, Any]]], Any] | None = None,
            array_hook: Callable[[list[Any]], Any] | None = None,
        ) -> None:
            """``object_hook``, if specified, will be called with the result
            of every JSON object decoded and its return value will be used in
            place of the given ``dict``.  This can be used to provide custom
            deserializations (e.g. to support JSON-RPC class hinting).

            ``object_pairs_hook``, if specified will be called with the result
            of every JSON object decoded with an ordered list of pairs.  The
            return value of ``object_pairs_hook`` will be used instead of the
            ``dict``.  This feature can be used to implement custom decoders.
            If ``object_hook`` is also defined, the ``object_pairs_hook`` takes
            priority.

            ``array_hook`` is an optional function that will be called with the
            result of any literal array decode (a ``list``). The return value of
            this function will be used instead of the ``list``. This feature can
            be used along ``object_pairs_hook`` to customize the resulting data
            structure - for example, by setting that to ``frozendict`` and
            ``array_hook`` to ``tuple``, one can get a deep immutable data
            structute from any JSON data.

            ``parse_float``, if specified, will be called with the string
            of every JSON float to be decoded. By default this is equivalent to
            float(num_str). This can be used to use another datatype or parser
            for JSON floats (e.g. decimal.Decimal).

            ``parse_int``, if specified, will be called with the string
            of every JSON int to be decoded. By default this is equivalent to
            int(num_str). This can be used to use another datatype or parser
            for JSON integers (e.g. float).

            ``parse_constant``, if specified, will be called with one of the
            following strings: -Infinity, Infinity, NaN.
            This can be used to raise an exception if invalid JSON numbers
            are encountered.

            If ``strict`` is false (true is the default), then control
            characters will be allowed inside strings.  Control characters in
            this context are those with character codes in the 0-31 range,
            including ``'\\t'`` (tab), ``'\\n'``, ``'\\r'`` and ``'\\0'``.
            """

    else:
        def __init__(
            self,
            *,
            object_hook: Callable[[dict[str, Any]], Any] | None = None,
            parse_float: Callable[[str], Any] | None = None,
            parse_int: Callable[[str], Any] | None = None,
            parse_constant: Callable[[str], Any] | None = None,
            strict: bool = True,
            object_pairs_hook: Callable[[list[tuple[str, Any]]], Any] | None = None,
        ) -> None:
            """``object_hook``, if specified, will be called with the result
            of every JSON object decoded and its return value will be used in
            place of the given ``dict``.  This can be used to provide custom
            deserializations (e.g. to support JSON-RPC class hinting).

            ``object_pairs_hook``, if specified will be called with the result
            of every JSON object decoded with an ordered list of pairs.  The
            return value of ``object_pairs_hook`` will be used instead of the
            ``dict``.  This feature can be used to implement custom decoders.
            If ``object_hook`` is also defined, the ``object_pairs_hook`` takes
            priority.

            ``parse_float``, if specified, will be called with the string
            of every JSON float to be decoded. By default this is equivalent to
            float(num_str). This can be used to use another datatype or parser
            for JSON floats (e.g. decimal.Decimal).

            ``parse_int``, if specified, will be called with the string
            of every JSON int to be decoded. By default this is equivalent to
            int(num_str). This can be used to use another datatype or parser
            for JSON integers (e.g. float).

            ``parse_constant``, if specified, will be called with one of the
            following strings: -Infinity, Infinity, NaN.
            This can be used to raise an exception if invalid JSON numbers
            are encountered.

            If ``strict`` is false (true is the default), then control
            characters will be allowed inside strings.  Control characters in
            this context are those with character codes in the 0-31 range,
            including ``'\\t'`` (tab), ``'\\n'``, ``'\\r'`` and ``'\\0'``.
            """

    def decode(self, s: str, _w: Callable[..., Any] = ...) -> Any:  # _w is undocumented
        """Return the Python representation of ``s`` (a ``str`` instance
        containing a JSON document).

        """

    def raw_decode(self, s: str, idx: int = 0) -> tuple[Any, int]:
        """Decode a JSON document from ``s`` (a ``str`` beginning with
        a JSON document) and return a 2-tuple of the Python
        representation and the index in ``s`` where the document ended.

        This can be used to decode a JSON document from a string that may
        have extraneous data at the end.

        """



import sys
from _typeshed import FileDescriptorLike, Unused
from collections.abc import Sequence
from struct import Struct
from typing import Any, Final, Literal

__all__ = ["ensure_running", "get_inherited_fds", "connect_to_new_process", "set_forkserver_preload"]

MAXFDS_TO_SEND: Final = 256
SIGNED_STRUCT: Final[Struct]

class ForkServer:
    if sys.version_info >= (3, 15):
        def set_forkserver_preload(
            self, modules_names: list[str], *, on_error: Literal["ignore", "warn", "fail"] = "ignore"
        ) -> None:
            """Set list of module names to try to load in forkserver process.

            The on_error parameter controls how import failures are handled:
            "ignore" (default) silently ignores failures, "warn" emits warnings,
            and "fail" raises exceptions breaking the forkserver context.
            """

    else:
        def set_forkserver_preload(self, modules_names: list[str]) -> None:
            """Set list of module names to try to load in forkserver process."""

    def get_inherited_fds(self) -> list[int] | None:
        """Return list of fds inherited from parent process.

        This returns None if the current process was not started by fork
        server.
        """

    def connect_to_new_process(self, fds: Sequence[int]) -> tuple[int, int]:
        """Request forkserver to create a child process.

        Returns a pair of fds (status_r, data_w).  The calling process can read
        the child process's pid and (eventually) its returncode from status_r.
        The calling process should write to data_w the pickled preparation and
        process data.
        """

    def ensure_running(self) -> None:
        """Make sure that a fork server is running.

        This can be called from any process.  Note that usually a child
        process will just reuse the forkserver started by its parent, so
        ensure_running() will do nothing.
        """

if sys.version_info >= (3, 15):
    def main(
        listener_fd: int | None,
        alive_r: FileDescriptorLike,
        preload: Sequence[str],
        main_path: str | None = None,
        sys_path: list[str] | None = None,
        *,
        sys_argv: list[str] | None = None,
        authkey_r: int | None = None,
        on_error: str = "ignore",
    ) -> None:
        """Run forkserver."""

elif sys.version_info >= (3, 14):
    # `sys_argv` parameter added in Python 3.14.3
    def main(
        listener_fd: int | None,
        alive_r: FileDescriptorLike,
        preload: Sequence[str],
        main_path: str | None = None,
        sys_path: list[str] | None = None,
        *,
        sys_argv: list[str] | None = None,
        authkey_r: int | None = None,
    ) -> None:
        """Run forkserver."""

elif sys.version_info >= (3, 13):
    # `sys_argv` parameter added in Python 3.13.12
    def main(
        listener_fd: int | None,
        alive_r: FileDescriptorLike,
        preload: Sequence[str],
        main_path: str | None = None,
        sys_path: list[str] | None = None,
        *,
        sys_argv: list[str] | None = None,
    ) -> None:
        """Run forkserver."""

else:
    def main(
        listener_fd: int | None,
        alive_r: FileDescriptorLike,
        preload: Sequence[str],
        main_path: str | None = None,
        sys_path: Unused = None,
    ) -> None:
        """Run forkserver."""

def read_signed(fd: int) -> Any: ...
def write_signed(fd: int, n: int) -> None: ...

_forkserver: ForkServer
ensure_running = _forkserver.ensure_running
"""Make sure that a fork server is running.

This can be called from any process.  Note that usually a child
process will just reuse the forkserver started by its parent, so
ensure_running() will do nothing.
"""

get_inherited_fds = _forkserver.get_inherited_fds
"""Return list of fds inherited from parent process.

This returns None if the current process was not started by fork
server.
"""

connect_to_new_process = _forkserver.connect_to_new_process
"""Request forkserver to create a child process.

Returns a pair of fds (status_r, data_w).  The calling process can read
the child process's pid and (eventually) its returncode from status_r.
The calling process should write to data_w the pickled preparation and
process data.
"""

set_forkserver_preload = _forkserver.set_forkserver_preload
"""Set list of module names to try to load in forkserver process.

The on_error parameter controls how import failures are handled:
"ignore" (default) silently ignores failures, "warn" emits warnings,
and "fail" raises exceptions breaking the forkserver context.
"""



"""Tools to analyze tasks running in asyncio programs."""

import sys
from collections.abc import Iterable
from enum import Enum
from typing import NamedTuple, SupportsIndex, type_check_only

@type_check_only
class _AwaitedInfo(NamedTuple):  # AwaitedInfo_Type from _remote_debugging
    thread_id: int
    awaited_by: list[_TaskInfo]

@type_check_only
class _TaskInfo(NamedTuple):  # TaskInfo_Type from _remote_debugging
    task_id: int
    task_name: str
    coroutine_stack: list[_CoroInfo]
    awaited_by: list[_CoroInfo]

@type_check_only
class _CoroInfo(NamedTuple):  # CoroInfo_Type from _remote_debugging
    call_stack: list[_FrameInfo]
    task_name: int | str

@type_check_only
class _FrameInfo(NamedTuple):  # FrameInfo_Type from _remote_debugging
    filename: str
    lineno: int
    funcname: str

class NodeType(Enum):
    COROUTINE = 1
    TASK = 2

class CycleFoundException(Exception):
    """Raised when there is a cycle when drawing the call tree."""

    cycles: list[list[int]]
    id2name: dict[int, str]
    def __init__(self, cycles: list[list[int]], id2name: dict[int, str]) -> None: ...

def get_all_awaited_by(pid: SupportsIndex) -> list[_AwaitedInfo]: ...
def build_async_tree(result: Iterable[_AwaitedInfo], task_emoji: str = "(T)", cor_emoji: str = "") -> list[list[str]]:
    """
    Build a list of strings for pretty-print an async call tree.

    The call tree is produced by `get_all_async_stacks()`, prefixing tasks
    with `task_emoji` and coroutine frames with `cor_emoji`.
    """

def build_task_table(result: Iterable[_AwaitedInfo]) -> list[list[int | str]]: ...

if sys.version_info >= (3, 14):
    def exit_with_permission_help_text() -> None:
        """
        Prints a message pointing to platform-specific permission help text and exits the program.
        This function is called when a PermissionError is encountered while trying
        to attach to a process.
        """

if sys.version_info >= (3, 15):
    def display_awaited_by_tasks_table(pid: SupportsIndex, retries: SupportsIndex = 3) -> None:
        """Build and print a table of all pending tasks under `pid`."""

    def display_awaited_by_tasks_tree(pid: SupportsIndex, retries: SupportsIndex = 3) -> None:
        """Build and print a tree of all pending tasks under `pid`."""

else:
    def display_awaited_by_tasks_table(pid: SupportsIndex) -> None:
        """Build and print a table of all pending tasks under `pid`."""

    def display_awaited_by_tasks_tree(pid: SupportsIndex) -> None:
        """Build and print a tree of all pending tasks under `pid`."""
